/**************************************************************************

   Fotoxx      edit photos and manage collections

   Copyright 2007-2015 Michael Cornelison
   Source URL: http://kornelix.com/fotoxx
   Contact: kornelix@posteo.de
   
   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program. If not, see http://www.gnu.org/licenses/.

***************************************************************************

   Fotoxx image edit functions - Mashup function

   Define a layout/background image. 
   Add other images as overlays on top of layout.
   Overlays have adjustable position, angle, size, transparency.

***************************************************************************/

#define EX extern                                                          //  enable extern declarations
#include "fotoxx.h"                                                        //  (variables in fotoxx.h are refs)

/**************************************************************************/

namespace mashup
{
   char        projectname[100];                                           //  project name
   char        projectfile[200], transpfile[220];                          //  project files /.../projectname.jpg
   editfunc    EFmashup;                                                   //  edit function data
   char        *layoutfile = 0;                                            //  background/layout image file
   int         Lww, Lhh;                                                   //  background/layout image size
   int         Fflatlayout;                                                //  layout file is flat RGB
   int         flatRGB[3];                                                 //  flat layout file RGB values
   int         Fupdall;                                                    //  flag, update entire layout
   int         Nimage;                                                     //  overlay image count
   int         Ntext;                                                      //  overlay text count
   int         Nline;                                                      //  overlay line count
   cchar       *focus;                                                     //  "image" "text" "line" ""
   int         focusii;                                                    //  image[ii] text[ii] line[ii] or -1
   int         Mradius = 50;                                               //  paint transparency mouse radius
   int         Mpowcen = 10;                                               //  paint transparency mouse center power
   int         Mpowedge = 0;                                               //  paint transparency mouse edge power
   int         Mgradual = 1;                                               //  paint transparency gradual or instant
   int         updxlo, updxhi, updylo, updyhi;                             //  update region in layout image
   zdialog     *zdimage;                                                   //  image edit dialog active
   zdialog     *zdtext;                                                    //  text edit dialog active
   zdialog     *zdline;                                                    //  line edit dialog active
   zdialog     *zdtransp;                                                  //  paint transparencies dialog active

   struct image_t {                                                        //  overlay image data
      char     *file;                                                      //  filespec
      PXM      *pxm1;                                                      //  PXM pixmap, 1x size
      PXM      *pxm2;                                                      //   " curr. size
      PXM      *pxm1f;                                                     //  info pixmap, if from select area   v.14.01
      PXM      *pxm2f;                                                     //   " curr. size
      float    scale;                                                      //  pxm2 scale, 0.0 to 1.0 = 1x
      float    theta;                                                      //  angle, -PI to +PI radians
      float    sinT, cosT;                                                 //  trig values for theta
      float    Btransp;                                                    //  base transparency, 0.0 to 1.0 = invisible
      int      Fblend;                                                     //  overall edge blend flag            v.15.02
      int      Lbdist, Rbdist;                                             //  left and right edge blend width
      int      Tbdist, Bbdist;                                             //  top and bottom edge blend width
      int      Vtranspcc;                                                  //  Vtransp data length 
      uint8    *Vtransp;                                                   //  var. transparency over image 
      int      ww1, hh1;                                                   //  image size at 1x scale
      int      ww2, hh2;                                                   //  image size at curr. scale
      int      px0, py0;                                                   //  image position in layout image
      int      pxlo, pxhi, pylo, pyhi;                                     //  image extent in layout image space
   };

   struct text_t {                                                         //  overlay text data
      textattr_t  attr;                                                    //  text, font, angle, outline ...
      int         ww, hh;                                                  //  image dimensions in layout image
      int         px0, py0;                                                //  image position in layout image
      int         pxlo, pxhi, pylo, pyhi;                                  //  extent of text image in layout image
   };

   struct line_t {                                                         //  overlay line/arrow data
      lineattr_t  attr;                                                    //  attributes (length, width, arrowheads)
      int         ww, hh;                                                  //  image dimensions in layout image
      int         px0, py0;                                                //  image position in layout image
      int         pxlo, pxhi, pylo, pyhi;                                  //  extent of line image in layout image
   };

   #define  maxmash 30                                                     //  max. images, text, lines
   image_t     image[maxmash];
   text_t      text[maxmash];
   line_t      line[maxmash];

   void  project();
   int   project_dialog_event(zdialog *zd, cchar *event);
   int   project_open();
   void  project_save();

   void  image_edit();
   int   image_dialog_event(zdialog *zd, cchar *event);
   void  paint_transp_dialog();
   int   paint_transp_dialog_event(zdialog *zd, cchar *event);
   void  mousefunc_transp();
   void  add_image();
   void  remove_image(int ii);
   void  flashimage();

   void  text_edit();
   int   text_dialog_event(zdialog *zd, cchar *event);
   void  add_text();
   void  remove_text(int ii);
   void  flashtext();

   void  line_edit();                                                      //  v.14.07
   int   line_dialog_event(zdialog *zd, cchar *event);
   void  add_line();
   void  remove_line(int ii);
   void  flashline();

   void  select(cchar *which, int ii);
   void  setlayoutupdatearea();
   void  mousefunc_layout();
   void  KBfunc(int key);
   void  E3update();
   void  * E3update_thread(void *);
   void  * E3update_wthreadimage(void *arg);
   void  * E3update_wthreadtext(void *arg);
   void  * E3update_wthreadline(void *arg);
}


void m_mashup(GtkWidget *, cchar *)
{
   using namespace mashup;

   zdialog     *zd;   
   cchar       *title = ZTX("Mashup layout and background image");
   int         zstat, nn, err, Nth = 0;
   cchar       *choice = 0, *file;
   float       *pix1;

   F1_help_topic = "mashup";
   if (checkpend("edit mods lock")) return;                                //  v.14.07
   
   *projectname = *projectfile = *transpfile = 0;
   zdimage = zdtransp = zdtext = zdline = 0;
   
   for (nn = 0; nn < maxmash; nn++) {                                      //  v.14.10
      memset(&image[nn],0,sizeof(image_t));
      memset(&text[nn],0,sizeof(text_t));
      memset(&line[nn],0,sizeof(line_t));
   }

/***
       _________________________________________
      |  Mashup layout and background image     |
      |                                         |
      |  (o) choose an image file               |
      |  (o) use current image file             |
      |  (o) open a Mashup project file         |
      |  (o) specify layout size and color      |
      |                                         |
      |                     [Proceed] [Cancel]  |
      |_________________________________________|

***/

   zd = zdialog_new("Mashup",Mwin,Bproceed,Bcancel,null);

   zdialog_add_widget(zd,"label","labtitle","dialog",title,"space=3");
   zdialog_add_widget(zd,"hbox","hbopt","dialog",0,"space=3");
   zdialog_add_widget(zd,"label","space","hbopt",0,"space=5");
   zdialog_add_widget(zd,"vbox","vbopt","hbopt");
   zdialog_add_widget(zd,"radio","choosefile","vbopt",ZTX("choose an image file"));
   zdialog_add_widget(zd,"radio","usecurrent","vbopt",ZTX("use current image file"));
   zdialog_add_widget(zd,"radio","openproject","vbopt",ZTX("open a Mashup project file"));
   zdialog_add_widget(zd,"radio","makelayout","vbopt",ZTX("specify layout size and color"));
   
   zdialog_stuff(zd,"choosefile",1);

   zdialog_run(zd,0,"save");
   zstat = zdialog_wait(zd);
   if (zstat != 1) {                                                       //  canceled
      zdialog_free(zd);
      return;
   }

   zdialog_fetch(zd,"choosefile",nn);
   if (nn) choice = "choosefile";
   zdialog_fetch(zd,"usecurrent",nn);
   if (nn) choice = "usecurrent";
   zdialog_fetch(zd,"openproject",nn);
   if (nn) choice = "openproject";
   zdialog_fetch(zd,"makelayout",nn);
   if (nn) choice = "makelayout";
   
   zdialog_free(zd);

   Lww = Lhh = 0;                                                          //  no layout file   
   Nimage = Ntext = Nline = 0;                                             //  no overlay images
   select("",-1);                                                          //  nothing selected

   if (strmatch(choice,"choosefile"))                                      //  open new curr. file >> layout
   {
      file = zgetfile(ZTX("choose layout file"),"file",curr_file,null);
      if (! file) return;
      err = f_open(file,Nth,0,1);
      if (err) return;
      project();
   }
   
   if (strmatch(choice,"usecurrent"))                                      //  curr. file >> layout
      project();
   
   if (strmatch(choice,"openproject")) {                                   //  project file >> layout & contents
      nn = project_open();
      if (! nn) m_mashup(0,0);                                             //  cancel or fail, start over
   }
   
   if (strmatch(choice,"makelayout")) {                                    //  create blank curr. file >> layout
      m_create(0,0);
      if (! curr_file) return;
      if (layoutfile) zfree(layoutfile);
      layoutfile = zstrdup(curr_file);
      project();
      Fflatlayout = 1;                                                     //  save blank file attributes
      pix1 = PXMpix(E1pxm,0,0); 
      flatRGB[0] = pix1[0];
      flatRGB[1] = pix1[1];
      flatRGB[2] = pix1[2];
   }
   
   return;
}


//  edit function dialog and overall steering dialog

void mashup::project()
{
   using namespace mashup;
   
   zdialog     *zd;

   EFmashup.menufunc = m_mashup;
   EFmashup.funcname = "mashup";                                           //  function name
   EFmashup.mousefunc = mousefunc_layout;                                  //  mouse function
   EFmashup.threadfunc = E3update_thread;                                  //  thread function
   EFmashup.zd = 0;

   if (! edit_setup(EFmashup)) return;                                     //  start new edit function
   
   Lww = E3pxm->ww;                                                        //  layout image dimensions
   Lhh = E3pxm->hh;
   Fupdall = 1;                                                            //  update entire layout

/***   
       ____________________________________________
      |          Mashup                            |
      |                                            |
      |  [Edit Images]   add or edit images        |
      |  [Edit Text]     add or edit text          |
      |  [Edit Line]     add or edit lines/arrows  |
      |  [Save Project]  save project data         |
      |  [Done]          project complete          |
      |  [Cancel]        cancel project            |
      |____________________________________________|
      
***/

   zd = zdialog_new("Mashup",Mwin,null);
   EFmashup.zd = zd;

   zdialog_add_widget(zd,"hbox","hb1","dialog",0,"space=3");
   zdialog_add_widget(zd,"vbox","vb1","hb1",0,"space=3|homog");
   zdialog_add_widget(zd,"vbox","vb2","hb1",0,"space=3|homog");
   zdialog_add_widget(zd,"button","editimage","vb1",ZTX("Edit Images"));
   zdialog_add_widget(zd,"button","edittext","vb1",ZTX("Edit Text"));
   zdialog_add_widget(zd,"button","editline","vb1",ZTX("Edit Line"));
   zdialog_add_widget(zd,"button","saveproject","vb1",ZTX("Save Project"));
   zdialog_add_widget(zd,"button","done","vb1",Bdone);
   zdialog_add_widget(zd,"button","cancel","vb1",Bcancel);
   zdialog_add_widget(zd,"label","labedim","vb2",ZTX("add or edit images"));
   zdialog_add_widget(zd,"label","labedtx","vb2",ZTX("add or edit text"));
   zdialog_add_widget(zd,"label","labedln","vb2",ZTX("add or edit lines/arrows"));
   zdialog_add_widget(zd,"label","labsavep","vb2",ZTX("save project data"));
   zdialog_add_widget(zd,"label","labdone","vb2",ZTX("project complete"));
   zdialog_add_widget(zd,"label","labcancel","vb2",ZTX("cancel project"));

   zdialog_run(zd,project_dialog_event);                                   //  start dialog
   takeMouse(mousefunc_layout,0);                                          //  capture mouse click/drag

   return;
}


//  project dialog event and completion function

int mashup::project_dialog_event(zdialog *zd, cchar *event)
{
   using namespace mashup;
   
   int      ii, yn;
   
   if (zd->zstat) event = "cancel";                                        //  window [x] button

   if (strmatch(event,"focus")) {
      takeMouse(mousefunc_layout,0);
      return 1;
   }
   
   if (strmatch(event,"editimage")) {
      zdialog_show(zd,0);
      image_edit();                                                        //  start edit images dialog
      zdialog_show(zd,1);
      return 1;
   }
   
   if (strmatch(event,"edittext")) {
      zdialog_show(zd,0);
      text_edit();                                                         //  start edit text dialog
      zdialog_show(zd,1);
      return 1;
   }
   
   if (strmatch(event,"editline")) {
      zdialog_show(zd,0);
      line_edit();                                                         //  start edit line dialog
      zdialog_show(zd,1);
      return 1;
   }
   
   if (strmatch(event,"saveproject")) {
      zdialog_show(zd,0);
      project_save();                                                      //  save project data
      zdialog_show(zd,1);                                                  //    for later re-edit
      return 1;
   }
   
   if (strmatch(event,"done")) {                                           //  image edit complete
      if (CEF->Fmods) {                                                    //  v.14.11
         yn = zmessageYN(Mwin,ZTX("save project file?"));                  //  offer to save project file
         if (yn) project_save();
      }

      edit_done(0);
      event = "done";                                                      //  lost with zdialog_free()        v.14.11
   }
   
   if (strmatch(event,"cancel")) {                                         //  image edit canceled
      edit_cancel(0);
      event = "cancel";                                                    //  lost with zdialog_free()        v.14.07

      if (*projectfile) {
         yn = zmessageYN(Mwin,ZTX("delete project file?"));                //  offer to delete project file    v.14.01
         if (yn) {
            remove(projectfile);
            if (*transpfile) remove(transpfile);
         }
      }

      if (Fflatlayout && layoutfile) {
         remove(layoutfile);
         zfree(layoutfile);
         layoutfile = 0;
         free_resources(0);
      }
   }
   
   if (strstr("done cancel",event))
   {
      if (zdimage) zdialog_free(zdimage);
      if (zdtransp) zdialog_free(zdtransp);
      if (zdtext) zdialog_free(zdtext);
      if (zdline) zdialog_free(zdline);

      for (ii = Nimage-1; ii >= 0; ii--)                                   //  free image memory
         remove_image(ii);
      for (ii = Ntext-1; ii >= 0; ii--)                                    //  free text memory
         remove_text(ii);
      for (ii = Nline-1; ii >= 0; ii--)                                    //  free line/arrow memory
         remove_line(ii);
   }

   return 1;
}


//  open and restore a saved mashup project
//  return 1 if success, 0 if fail or cancel

int mashup::project_open()
{
   using namespace mashup;

   int      ii = 0, nn, err, Nth = 0;
   int      ww1, hh1, ww2, hh2;
   int      nfid = -1, cc, cc2;
   char     *pp, *pp2;
   char     *infofile = 0, buff[maxfcc];
   FILE     *fid;
   STATB    sbuff;

   for (ii = Nimage-1; ii >= 0; ii--)                                      //  free image memory                  v.14.10
      remove_image(ii);
   for (ii = Ntext-1; ii >= 0; ii--)                                       //  free text memory
      remove_text(ii);
   for (ii = Nline-1; ii >= 0; ii--)                                       //  free line/arrow memory
      remove_line(ii);  

   Nimage = Ntext = Nline = 0;

   if (layoutfile) zfree(layoutfile);
   layoutfile = 0;
   textattr_t  *txattr = 0;
   lineattr_t  *lnattr = 0;

   snprintf(projectfile,200,"%s",mashup_dirk);                             //  mashup projects directory
   pp = zgetfile(ZTX("Open Project"),"file",projectfile,null);             //  get project file from user
   if (! pp) return 0;
   pp2 = strstr(pp,"_transparencies");                                     //  if user opened a transparencies file
   if (pp2) *pp2 = 0;                                                      //  fix the problem

   strncpy0(projectfile,pp,200);                                           //  project file
   strncpy0(transpfile,pp,200);
   strcat(transpfile,"_transparencies");                                   //  corresp. transparencies file
   zfree(pp);
   
   //  read project file and validate all data

   fid = fopen(projectfile,"r");                                           //  open project file
   if (! fid) goto failure;
   
   pp = fgets_trim(buff,maxfcc,fid);                                       //  read layout record
   if (! pp) goto badproject;
   
   if (strmatchN(pp,"layout:",7))                                          //  layout: /.../filename.jpg
   {
      pp += 7;
      while (*pp == ' ') pp++;

      layoutfile = zstrdup(pp);
      err = stat(layoutfile,&sbuff);
      if (err) {
         zmessageACK(Mwin,0,ZTX("layout image file missing: \n %s"),layoutfile);
         goto badproject;
      }
   }
   
   else if (strmatchN(pp,"flatlayout:",11))                                //  flatlayout: www hhh RGB: rr/gg/bb
   {
      nn = sscanf(buff,"flatlayout: %d %d RGB: %d/%d/%d \n",
                   &Lww,&Lhh,&flatRGB[0],&flatRGB[1],&flatRGB[2]);
      if (nn != 5) goto badproject;
      
      if (curr_dirk) layoutfile = zstrdup(curr_dirk,200);
      else layoutfile = zstrdup(topdirks[0],200);
      pp = strrchr(projectfile,'/');
      strcat(layoutfile,pp);
      strcat(layoutfile,".jpg");
      err = create_blank_file(layoutfile,Lww,Lhh,flatRGB);
      if (err) goto badproject;
      Fflatlayout = 1;
   }

   else goto badproject;

   //  read any number of overlay image records
   
   while (true)
   {
      pp = fgets_trim(buff,maxfcc,fid);
      if (! pp) break;
      if (strmatchN(pp,"end",3)) break;
      
      if (strmatchN(pp,"image:",6))                                        //  image: /.../filename.jpg
      {
         ii = Nimage++;                                                    //  new image
         cc = sizeof(image_t);                                             //  clear image data
         memset(&image[ii],0,cc);

         image[ii].Vtransp = 0;                                            //  initz. no var. transparency
         image[ii].Vtranspcc = 0;

         pp += 6;
         while (*pp == ' ') pp++;
         image[ii].file = zstrdup(pp);                                     //  overlay image file

         err = stat(image[ii].file,&sbuff);
         if (err) {
            zmessageACK(Mwin,0,ZTX("overlay image file missing: \n %s"),image[ii].file);
            goto cleanupErr;
         }

         if (strstr(image[ii].file,saved_areas_dirk)) {                    //  image is from saved select area
            infofile = zstrdup(image[ii].file,8);
            pp = strstr(infofile,".tiff");
            if (! pp) pp = infofile + strlen(infofile);                    //  check area info file exists        v.14.01
            strcpy(pp,".info.tiff");
            err = stat(infofile,&sbuff);
            zfree(infofile);
            if (err) {
               zmessageACK(Mwin,0,ZTX("area info file missing: \n %s"),infofile);
               goto cleanupErr;
            }
         }

         image[ii].pxm1 = PXM_load(image[ii].file,1);                      //  validate file (with diagnostic)
         if (! image[ii].pxm1) goto cleanupErr;

         PXM_fixblue(image[ii].pxm1);                                      //  RGB 0/0/0 pixel >> 0/0/1 for vpixel
         
         if (strstr(image[ii].file,saved_areas_dirk)) {                    //  image is from saved select area
            infofile = zstrdup(image[ii].file,8);
            pp = strstr(infofile,".tiff");
            if (! pp) pp = infofile + strlen(infofile);                    //  load corresp. area info image      v.14.01
            strcpy(pp,".info.tiff");
            image[ii].pxm1f = PXM_load(infofile,1);
            zfree(infofile);
            if (! image[ii].pxm1f) goto cleanupErr;
         }

         continue;
      }

      if (strmatchN(pp,"position:",9)) {                                   //  position: xxx yyy
         nn = sscanf(pp,"position: %d %d",&image[ii].px0,&image[ii].py0);
         if (nn != 2) goto badproject;
         continue;
      }

      if (strmatchN(pp,"scale:",6)) {                                      //  scale: n.nnn
         nn = sscanf(pp,"scale: %f",&image[ii].scale);
         if (nn != 1) goto badproject;
         continue;
      }

      if (strmatchN(pp,"theta:",6)) {                                      //  theta: n.nnn
         nn = sscanf(pp,"theta: %f",&image[ii].theta);
         if (nn != 1) goto badproject;
         continue;
      }

      if (strmatchN(pp,"Btransp:",8)) {                                    //  Btransp: 0.nnn
         nn = sscanf(pp,"Btransp: %f",&image[ii].Btransp);
         if (nn != 1) goto badproject;
         continue;
      }

      if (strmatchN(pp,"Lbdist:",7)) {                                     //  Lbdist: nnn                        v.15.02
         nn = sscanf(pp,"Lbdist: %d",&image[ii].Lbdist);
         if (nn != 1) goto badproject;
         continue;
      }

      if (strmatchN(pp,"Rbdist:",7)) {                                     //  Rbdist: nnn
         nn = sscanf(pp,"Rbdist: %d",&image[ii].Rbdist);
         if (nn != 1) goto badproject;
         continue;
      }

      if (strmatchN(pp,"Tbdist:",7)) {                                     //  Tbdist: nnn
         nn = sscanf(pp,"Tbdist: %d",&image[ii].Tbdist);
         if (nn != 1) goto badproject;
         continue;
      }

      if (strmatchN(pp,"Bbdist:",7)) {                                     //  Bbdist: nnn
         nn = sscanf(pp,"Bbdist: %d",&image[ii].Bbdist);
         if (nn != 1) goto badproject;
         continue;
      }

      if (strmatchN(pp,"Vtranspcc:",10)) {                                 //  Vtranspcc: nnnn
         nn = sscanf(pp,"Vtranspcc: %d",&image[ii].Vtranspcc);
         if (nn != 1) goto badproject;
         continue;
      }
   }

   //  read any number of text records
   
   while (true)
   {
      pp = fgets_trim(buff,maxfcc,fid);
      if (! pp) break;
      if (strmatchN(pp,"end",3)) break;
      
      if (strmatchN(pp,"text:",5))                                         //  text: this is my text ...
      {
         ii = Ntext++;
         txattr = &text[ii].attr;
         pp += 5;
         while (*pp == ' ') pp++;
         repl_Nstrs(pp,txattr->text,"\\n","\n",null);                      //  replace "\n" with newline
         continue;
      }
      
      if (strmatchN(pp,"position:",9)) {                                   //  position: xx yy
         nn = sscanf(pp,"position: %d %d",&text[ii].px0,&text[ii].py0);
         if (nn != 2) goto badproject;
         continue;
      }
      
      if (strmatchN(pp,"font:",5)) {                                       //  font: free sans  size: nn
         pp += 5;
         while (*pp == ' ') pp++;
         pp2 = strstr(pp,"size:");
         if (! pp2) goto badproject;
         *pp2 = 0;
         pp2 += 5;
         strncpy0(txattr->font,pp,80);
         txattr->size = atoi(pp2);
         if (txattr->size < 8) goto badproject;
         continue;
      }

      if (strmatchN(pp,"attributes:",11)) {                                //  attributes
         txattr = &text[ii].attr;
         nn = sscanf(pp,"attributes: %f  %s %s %s %s  %d %d %d %d  %d %d %d",
                  &txattr->angle, txattr->color[0], txattr->color[1], txattr->color[2], txattr->color[3],
                  &txattr->transp[0], &txattr->transp[1], &txattr->transp[2], &txattr->transp[3],
                  &txattr->towidth, &txattr->shwidth, &txattr->shangle);
         if (nn != 12) goto badproject;
      }
   }
   
   //  read any number of line/arrow records                               //  v.14.07
   
   while (true)
   {
      pp = fgets_trim(buff,maxfcc,fid);
      if (! pp) break;
      if (strmatchN(pp,"end",3)) break;

      if (strmatchN(pp,"line:",5)) {                                       //  line:
         ii = Nline++;
         continue;
      }
      
      if (strmatchN(pp,"position:",9)) {                                   //  position: xx yy
         nn = sscanf(pp,"position: %d %d",&line[ii].px0,&line[ii].py0);
         if (nn != 2) goto badproject;
         continue;
      }
      
      if (strmatchN(pp,"attributes:",11)) {                                //  attributes:
         lnattr = &line[ii].attr;
         nn = sscanf(pp,"attributes: %d %d %d %d %f  %s %s %s %s  %d %d %d %d  %d %d %d",
                  &lnattr->length, &lnattr->width, &lnattr->larrow, &lnattr->rarrow, &lnattr->angle,
                  lnattr->color[0], lnattr->color[1], lnattr->color[2], lnattr->color[3],
                  &lnattr->transp[0], &lnattr->transp[1], &lnattr->transp[2], &lnattr->transp[3],
                  &lnattr->towidth, &lnattr->shwidth, &lnattr->shangle);
         if (nn != 16) goto badproject;
      }
   }

   fclose(fid);
   fid = 0;
   
   //  read binary transparency file if any
   
   for (cc = ii = 0; ii < Nimage; ii++)                                    //  sum transparency data 
      cc += image[ii].Vtranspcc;
   
   if (cc)
   {
      nfid = open(transpfile,O_RDONLY);
      if (nfid < 0) goto failure;
      
      for (ii = 0; ii < Nimage; ii++)                                      //  read transparency data 
      {
         cc = image[ii].Vtranspcc;
         if (cc) {
            image[ii].Vtransp = (uint8 *) zmalloc(cc);
            cc2 = read(nfid,image[ii].Vtransp,cc);
            if (cc2 != cc) goto badproject;
         }
         else image[ii].Vtransp = 0;
      }

      close(nfid);
      nfid = -1;
   }
   
   //  create active project from loaded data

   err = f_open(layoutfile,Nth,0,1);                                       //  open layout image file (with diagnostic)
   if (err) goto cleanupErr;

   project();                                                              //  start mashup project

   //  generate overlay images
   
   for (ii = 0; ii < Nimage; ii++)
   {
      image[ii].sinT = sin(image[ii].theta);
      image[ii].cosT = cos(image[ii].theta);

      ww1 = image[ii].pxm1->ww;                                            //  image size at 1x scale
      hh1 = image[ii].pxm1->hh;
      ww2 = image[ii].scale * ww1;                                         //  image size in layout
      hh2 = image[ii].scale * hh1;
      image[ii].pxm2 = PXM_rescale(image[ii].pxm1,ww2,hh2);                //  rescaled image in layout
      image[ii].ww1 = ww1;                                                 //  1x image size
      image[ii].hh1 = hh1;
      image[ii].ww2 = ww2;                                                 //  image size in layout
      image[ii].hh2 = hh2;
      if (image[ii].pxm1f)
         image[ii].pxm2f = PXM_rescale(image[ii].pxm1f,ww2,hh2);           //  info image, if select area         v.14.01
      else image[ii].pxm2f = 0;

      nn = image[ii].Lbdist + image[ii].Rbdist                             //  overal edge blend flag             v.15.02
         + image[ii].Tbdist + image[ii].Bbdist;
      if (nn) nn = 1;
      image[ii].Fblend = nn;

      select("image",ii);
      setlayoutupdatearea();                                               //  set layout update area
   }

   //  generate text images
   
   for (ii = 0; ii < Ntext; ii++)
   {
      gentext(&text[ii].attr);                                             //  make image from text and attributes
      text[ii].ww = text[ii].attr.pxm_text->ww;                            //  text image size in layout
      text[ii].hh = text[ii].attr.pxm_text->hh;
      
      select("text",ii);
      setlayoutupdatearea();                                               //  set layout update area
   }

   //  generate line/arrow images

   for (ii = 0; ii < Nline; ii++)                                          //  load overlay line/arrow images     v.14.07
   {
      genline(&line[ii].attr);                                             //  make image from line attributes
      line[ii].ww = line[ii].attr.pxm_line->ww;                            //  line image size in layout
      line[ii].hh = line[ii].attr.pxm_line->hh;

      select("line",ii);
      setlayoutupdatearea();                                               //  set layout update area
   }
   
   select("",-1);                                                          //  nothing selected
   E3update();                                                             //  update layout composite image
   
   zfree(layoutfile);
   layoutfile = 0;
   return 1;                                                               //  success return

failure:
   if (fid) fclose(fid);
   if (nfid > -1) close(nfid);
   zmessageACK(Mwin,0,"%s \n %s \n %s",projectfile,transpfile,strerror(errno));
   goto cleanupErr;

badproject:                                                                //  project file had a problem
   if (fid) fclose(fid);
   if (nfid > -1) close(nfid);
   zmessageACK(Mwin,0,ZTX("project file is defective"));

cleanupErr:
   err = remove(projectfile);                                              //  delete bad project files
   err = remove(transpfile);
   if (layoutfile) zfree(layoutfile);
   layoutfile = 0;
   for (ii = Nimage-1; ii >= 0; ii--)                                      //  free image memory
      remove_image(ii);
   for (ii = Ntext-1; ii >= 0; ii--)                                       //  free text memory
      remove_text(ii);
   for (ii = Nline-1; ii >= 0; ii--)                                       //  free line/arrow memory
      remove_line(ii);
   return 0;                                                               //  failure return
}


//  save a mashup project so that it can be edited again later

void mashup::project_save()
{
   using namespace mashup;

   textattr_t  txattr;
   lineattr_t  lnattr;
   char        *pp, *pp2, text2[1200];
   FILE        *fid;
   int         nfid, ii, cc, cc2;
   
   pp = strrchr(curr_file,'/');                                            //  curr. file name >> default project name
   strncpy0(projectname,pp+1,100);
   pp = strrchr(projectname,'.');                                          //  remove .ext
   if (pp && strlen(pp) < 6) *pp = 0;
   snprintf(projectfile,200,"%s/%s",mashup_dirk,projectname);

   pp = zgetfile(ZTX("Save Project"),"save",projectfile,null);             //  get project name from user
   if (! pp) return;
   pp2 = strstr(pp,"_transparencies");                                     //  if user opened a transparencies file
   if (pp2) *pp2 = 0;                                                      //  fix the problem
   strncpy0(projectfile,pp,200);                                           //  project file
   strncpy0(transpfile,pp,200);
   strcat(transpfile,"_transparencies");                                   //  corresp. transparencies file
   zfree(pp);

   fid = fopen(projectfile,"w");
   if (! fid) goto failure;
   
   if (Fflatlayout)                                                        //  write attributes of flat layout
      fprintf(fid,"flatlayout: %d %d RGB: %d/%d/%d \n",
                     Lww,Lhh,flatRGB[0],flatRGB[1],flatRGB[2]);

   else fprintf(fid,"layout: %s\n",curr_file);                             //  write layout file
   
   for (ii = 0; ii < Nimage; ii++)                                         //  write image data
   {
      fprintf(fid,"image: %s\n",image[ii].file);
      fprintf(fid,"position: %d %d \n", image[ii].px0, image[ii].py0);
      fprintf(fid,"scale: %.4f \n",image[ii].scale);
      fprintf(fid,"theta: %.4f \n",image[ii].theta);
      fprintf(fid,"Btransp: %.4f \n",image[ii].Btransp);
      fprintf(fid,"Lbdist: %d \n",image[ii].Lbdist);                       //  v.15.02
      fprintf(fid,"Rbdist: %d \n",image[ii].Rbdist);
      fprintf(fid,"Tbdist: %d \n",image[ii].Tbdist);
      fprintf(fid,"Bbdist: %d \n",image[ii].Bbdist);
      fprintf(fid,"Vtranspcc: %d \n",image[ii].Vtranspcc);
   }
   
   fprintf(fid,"end\n");

   for (ii = 0; ii < Ntext; ii++)                                          //  write text data
   {
      repl_Nstrs(text[ii].attr.text, text2,"\n","\\n",null);
      fprintf(fid,"text: %s\n",text2);
      fprintf(fid,"position: %d %d \n", text[ii].px0, text[ii].py0);
      txattr = text[ii].attr;
      fprintf(fid,"font: %s  size: %d \n", txattr.font, txattr.size);
      fprintf(fid,"attributes: %.4f  %s %s %s %s  %d %d %d %d  %d %d %d \n", 
               txattr.angle, txattr.color[0], txattr.color[1], txattr.color[2], txattr.color[3],
               txattr.transp[0], txattr.transp[1], txattr.transp[2], txattr.transp[3],
               txattr.towidth, txattr.shwidth, txattr.shangle);
   }

   fprintf(fid,"end\n");

   for (ii = 0; ii < Nline; ii++)                                          //  write line/arrow data              v.14.07
   {
      fprintf(fid,"line:\n");
      fprintf(fid,"position: %d %d \n", line[ii].px0, line[ii].py0);
      lnattr = line[ii].attr;
      fprintf(fid,"attributes: %d %d %d %d %.4f  %s %s %s %s  %d %d %d %d  %d %d %d \n", 
               lnattr.length, lnattr.width, lnattr.larrow, lnattr.rarrow, lnattr.angle, 
               lnattr.color[0], lnattr.color[1], lnattr.color[2], lnattr.color[3],
               lnattr.transp[0], lnattr.transp[1], lnattr.transp[2], lnattr.transp[3],
               lnattr.towidth, lnattr.shwidth, lnattr.shangle);
   }

   fprintf(fid,"end\n");

   fclose(fid);

   for (cc = ii = 0; ii < Nimage; ii++)                                    //  sum transparency data 
      cc += image[ii].Vtranspcc;

   if (cc == 0) {
      remove(transpfile);                                                  //  no transparencies, remove poss. file
      return;
   }

   nfid = open(transpfile,O_WRONLY|O_CREAT|O_TRUNC,0640);                  //  write transparencies to binary file
   if (nfid < 0) goto failure;

   for (ii = 0; ii < Nimage; ii++)
   {
      cc = image[ii].Vtranspcc;
      if (! cc) continue;
      cc2 = write(nfid,image[ii].Vtransp,cc);
      if (cc2 != cc) goto failure;
   }

   close(nfid);
   return;

failure:
   zmessageACK(Mwin,0,strerror(errno));
   return;
}


//  mashup image edit function

void mashup::image_edit()
{
   using namespace mashup;

   cchar       *tipmess = ZTX("Click image to select, drag image to move.");
   
   select("",-1);                                                          //  nothing selected

/***
          _______________________________________________
         |              Edit Images                      |
         |                                               |
         |  Click image to select, drag image to move.   |
         |                                               |
         |  Current image: filename.jpg                  |
         |  Scale [0.345|v]  [ 1.0 ]                     |
         |  Angle [12.3|v]                               |
         |  Stacking Order  [raise] [lower]              |
         |  Base Transparency [0.5|v]                    |
         |  Variable Transparency  [paint]               |
         |  Edge Blend   Left    [123|v]                 |
         |               Right   [  0|v]                 |
         |                Top    [234|v]                 |
         |               Bottom  [  0|v]                 |
         |                                               |
         |                        [Add] [Delete] [Done]  |
         |_______________________________________________|
         
***/

   zdimage = zdialog_new("Edit Images",Mwin,Badd,Bdelete,Bdone,null);
   zdialog *zd = zdimage;

   zdialog_add_widget(zd,"hbox","hbtip","dialog",0,"space=3");
   zdialog_add_widget(zd,"label","labtip","hbtip",tipmess,"space=3");

   zdialog_add_widget(zd,"hbox","hbfile","dialog",0,"space=3");
   zdialog_add_widget(zd,"label","labfile","hbfile",ZTX("current image:"),"space=3");
   zdialog_add_widget(zd,"label","currfile","hbfile",0,"space=3");
   zdialog_add_widget(zd,"vbox","space","dialog",0,"space=5");      
   
   zdialog_add_widget(zd,"hbox","hbscale","dialog");
   zdialog_add_widget(zd,"label","labscale","hbscale",ZTX("Scale"),"space=3");
   zdialog_add_widget(zd,"spin","scale","hbscale","0.02|2.0|0.001|0.3");
   zdialog_add_widget(zd,"button","scale1x","hbscale"," 1.0 ","space=10");

   zdialog_add_widget(zd,"hbox","hbangle","dialog");
   zdialog_add_widget(zd,"label","labangle","hbangle",Bangle,"space=3");
   zdialog_add_widget(zd,"spin","angle","hbangle","-180|180|0.1|0");
   
   zdialog_add_widget(zd,"hbox","hbstack","dialog");
   zdialog_add_widget(zd,"label","labstack","hbstack",ZTX("Stacking Order"),"space=3");
   zdialog_add_widget(zd,"button","raise","hbstack",ZTX("raise"),"space=5");
   zdialog_add_widget(zd,"button","lower","hbstack",ZTX("lower"),"space=5");

   zdialog_add_widget(zd,"hbox","hbbtransp","dialog");
   zdialog_add_widget(zd,"label","labbtr","hbbtransp",ZTX("Base Transparency"),"space=3");
   zdialog_add_widget(zd,"spin","Btransp","hbbtransp","0|1.0|0.01|0");

   zdialog_add_widget(zd,"hbox","hbvtransp","dialog");
   zdialog_add_widget(zd,"label","labvtr","hbvtransp",ZTX("Var. Transparency"),"space=3");
   zdialog_add_widget(zd,"button","Vtransp","hbvtransp",ZTX("paint"),"space=5");

   zdialog_add_widget(zd,"hbox","hbblend","dialog",0,"space=5");           //  edge blend widths                  v.15.02
   zdialog_add_widget(zd,"vbox","vbblend1","hbblend",0,"space=3|homog");
   zdialog_add_widget(zd,"label","space","hbblend",0,"space=5");
   zdialog_add_widget(zd,"vbox","vbblend2","hbblend",0,"homog");
   zdialog_add_widget(zd,"label","space","hbblend",0,"space=5");
   zdialog_add_widget(zd,"vbox","vbblend3","hbblend",0,"homog");
   zdialog_add_widget(zd,"label","lablend","vbblend1",ZTX("Edge Blend"));
   zdialog_add_widget(zd,"label","space","vbblend1");
   zdialog_add_widget(zd,"label","space","vbblend1");
   zdialog_add_widget(zd,"label","space","vbblend1");
   zdialog_add_widget(zd,"label","labLbdist","vbblend2",Bleft);
   zdialog_add_widget(zd,"label","labRbdist","vbblend2",Bright);
   zdialog_add_widget(zd,"label","labTbdist","vbblend2",Btop);
   zdialog_add_widget(zd,"label","labBbdist","vbblend2",Bbottom);
   zdialog_add_widget(zd,"spin","Lbdist","vbblend3","0|999|1|0");
   zdialog_add_widget(zd,"spin","Rbdist","vbblend3","0|999|1|0");
   zdialog_add_widget(zd,"spin","Tbdist","vbblend3","0|999|1|0");
   zdialog_add_widget(zd,"spin","Bbdist","vbblend3","0|999|1|0");

   zdialog_add_ttip(zd,Badd,ZTX("add images to layout"));                  //  v.14.08

   zdialog_run(zd,image_dialog_event,"save");                              //  run dialog - parallel
   zdialog_wait(zd);
   zdialog_free(zd);                                                       //  v.14.06
   return;
}


//  image dialog event and completion function

int mashup::image_dialog_event(zdialog *zd, cchar *event)
{
   using namespace mashup;
   
   int      ii, bd, ww1, hh1, ww2, hh2;
   float    angle, rad = PI / 180.0;
   image_t  temp;
   
   if (strmatch(event,"focus")) {                                          //  get mouse ownership
      takeMouse(mousefunc_layout,0);
      return 1;
   }
   
   if (zd->zstat)
   {
      if (zd->zstat == 1) {                                                //  add image
         zd->zstat = 0;
         add_image();
         return 1;
      }
      
      if (zd->zstat == 2) {                                                //  remove image
         zd->zstat = 0;
         if (! strmatch(focus,"image")) return 1;
         ii = focusii;
         remove_image(ii);
         return 1;
      }
      
      zdimage = 0;                                                         //  done or [x] kill

      if (zdtransp) {
         zdialog_free(zdtransp);
         zdtransp = 0;
      }

      return 1;
   }

   if (strmatch(event,"Vtransp")) {                                        //  paint variable transparency dialog
      paint_transp_dialog();
      return 1;
   }

   if (! strmatch(focus,"image")) return 1;                                //  get selected image
   ii = focusii;

   if (strmatch(event,"scale"))                                            //  resize 
   {
      zdialog_fetch(zd,"scale",image[ii].scale);                           //  new image scale 
      ww1 = image[ii].ww1;                                                 //  image size, 1x
      hh1 = image[ii].hh1;
      ww2 = ww1 * image[ii].scale;                                         //  image size at scale
      hh2 = hh1 * image[ii].scale;
      PXM_free(image[ii].pxm2);
      image[ii].pxm2 = PXM_rescale(image[ii].pxm1,ww2,hh2);                //  rescale image
      image[ii].ww2 = ww2;
      image[ii].hh2 = hh2;
      PXM_free(image[ii].pxm2f);
      if (image[ii].pxm1f) 
         image[ii].pxm2f = PXM_rescale(image[ii].pxm1f,ww2,hh2);           //  rescale area info image            v.14.01
   }
   
   if (strmatch(event,"scale1x"))                                          //  resize to 1x                       v.14.12
   {
      zdialog_stuff(zd,"scale",1.0);
      image[ii].scale = 1.0;
      PXM_free(image[ii].pxm2);
      image[ii].pxm2 = PXM_copy(image[ii].pxm1);
      image[ii].ww2 = image[ii].ww1;
      image[ii].hh2 = image[ii].hh1;
      PXM_free(image[ii].pxm2f);
      if (image[ii].pxm1f) 
         image[ii].pxm2f = PXM_copy(image[ii].pxm1f);                      //  rescale area info image
   }
   
   if (strmatch(event,"angle"))                                            //  rotate
   {
      zdialog_fetch(zd,"angle",angle);
      angle = angle * rad;
      image[ii].theta = angle;
      image[ii].cosT = cos(angle);
      image[ii].sinT = sin(angle);
   }

   if (strmatch(event,"Btransp"))                                          //  base transparency, 0 to 1.0 = invisible
      zdialog_fetch(zd,"Btransp",image[ii].Btransp);

   if (strmatch(event,"Lbdist"))                                           //  left edge blend width              v.15.02
      zdialog_fetch(zd,"Lbdist",image[ii].Lbdist);

   if (strmatch(event,"Rbdist"))                                           //  right edge blend width             v.15.02
      zdialog_fetch(zd,"Rbdist",image[ii].Rbdist);

   if (strmatch(event,"Tbdist"))                                           //  top edge blend width               v.15.02
      zdialog_fetch(zd,"Tbdist",image[ii].Tbdist);

   if (strmatch(event,"Bbdist"))                                           //  bottom edge blend width            v.15.02
      zdialog_fetch(zd,"Bbdist",image[ii].Bbdist);
   
   if (strstr(event,"bdist")) {                                            //  overall edge blend flag            v.15.02
      bd = image[ii].Lbdist + image[ii].Rbdist 
         + image[ii].Tbdist + image[ii].Bbdist;
      if (bd) bd = 1;
      image[ii].Fblend = bd;
   }

   if (strmatch(event,"raise")) {                                          //  raise image in stacking order      v.14.06
      if (ii == Nimage - 1) return 1;
      temp = image[ii+1];
      image[ii+1] = image[ii];
      image[ii] = temp;
      ii = focusii = ii + 1;
   }        

   if (strmatch(event,"lower")) {                                          //  lower image in stacking order      v.14.06
      if (ii == 0) return 1;
      temp = image[ii-1];
      image[ii-1] = image[ii];
      image[ii] = temp;
      ii = focusii = ii - 1;
   }

   setlayoutupdatearea();                                                  //  set layout update area
   zmainloop();                                                            //  v.14.07
   E3update();                                                             //  update layout composite image

   return 1;
}


//  dialog to initiate painting image var. transparency

void mashup::paint_transp_dialog()
{
   using namespace mashup;

   cchar    *ptitle = ZTX("Paint Image Transparencies");

/***
          ________________________________
         |  Paint Image Transparencies    |
         |                                |
         | Radius [123|^]   Gradual [x]   |
         | Power Center [100|^]           |
         |         Edge [ 10|^]           |
         |                        [done]  |
         |________________________________|
         
***/

   zdtransp = zdialog_new(ptitle,Mwin,Bdone,null);
   zdialog_add_widget(zdtransp,"hbox","hbrad","dialog",0,"space=3");
   zdialog_add_widget(zdtransp,"label","labrad","hbrad",Bradius,"space=5");
   zdialog_add_widget(zdtransp,"spin","radius","hbrad","5|300|1|100");
   zdialog_add_widget(zdtransp,"check","gradual","hbrad",ZTX("Gradual"),"space=10");
   zdialog_add_widget(zdtransp,"hbox","hbpow","dialog",0,"space=3");
   zdialog_add_widget(zdtransp,"vbox","vbpow1","hbpow",0,"space=3");
   zdialog_add_widget(zdtransp,"vbox","vbpow2","hbpow",0,"space=3");
   zdialog_add_widget(zdtransp,"vbox","vbpow3","hbpow",0,"space=3");
   zdialog_add_widget(zdtransp,"label","labpower","vbpow1",ZTX("Power"),"space=5");
   zdialog_add_widget(zdtransp,"label","labcen","vbpow2",Bcenter,"space=5");
   zdialog_add_widget(zdtransp,"spin","center","vbpow3","0|100|1|10");
   zdialog_add_widget(zdtransp,"label","labedge","vbpow2",Bedge,"space=5");
   zdialog_add_widget(zdtransp,"spin","edge","vbpow3","0|100|1|3");

   zdialog_stuff(zdtransp,"radius",Mradius);
   zdialog_stuff(zdtransp,"radius",Mradius);
   zdialog_stuff(zdtransp,"gradual",Mgradual);
   zdialog_stuff(zdtransp,"edge",Mpowedge);

   zdialog_run(zdtransp,paint_transp_dialog_event,"save");
   takeMouse(mousefunc_transp,0);
   return;
}


//  zdialog event and completion function

int mashup::paint_transp_dialog_event(zdialog *zd, cchar *event)
{
   using namespace mashup;
   
   if (zd->zstat) {                                                        //  done or cancel
      zdialog_free(zd);
      zdtransp = 0;
      takeMouse(mousefunc_layout,0);
      return 1;
   }
   
   if (strmatch(event,"focus")) {                                          //  take mouse ownership
      takeMouse(mousefunc_transp,0);
      return 1;
   }
   
   if (strmatch(event,"radius"))                                           //  mouse radius value
      zdialog_fetch(zd,"radius",Mradius);
   
   if (strmatch(event,"gradual"))                                          //  gradual/instant paint option
      zdialog_fetch(zd,"gradual",Mgradual);
   
   if (strmatch(event,"center"))                                           //  mouse center power
      zdialog_fetch(zd,"center",Mpowcen);
   
   if (strmatch(event,"edge"))                                             //  mouse edge power
      zdialog_fetch(zd,"edge",Mpowedge);
   
   return 1;
}


//  mouse function for painting image var. transparency

void mashup::mousefunc_transp()
{
   using namespace mashup;

   int         ii, Mii, jj, cc, mx, my;
   int         distx, disty, dist, mindist;
   int         pxlo, pylo, pxhi, pyhi;
   int         cenx, ceny;
   int         mrad = Mradius, mrect = 2 * mrad;
   int         ww1, hh1, ww2, hh2, transp, mtransp;
   int         dx, dy, qx, qy, rx, ry;
   float       sinT, cosT, px0, py0, px1, py1;

   px1 = py1 = -1;                                                         //  no mouse position in image

   if (! strmatch(focus,"image")) {                                        //  if no image focus,
      draw_mousearc(0,0,0,0,1);                                            //    no mouse circle
      gdk_window_set_cursor(gdkwin,0);
   }

   mx = Mxposn;                                                            //  mouse position in layout
   my = Myposn;

   if (LMclick)                                                            //  left click
   {
      LMclick = 0;                                                         //  stop main window click after return

      mx = Mxclick;                                                        //  click position
      my = Myclick;

      Mii = -1;                                                            //  no selected image
      mindist = 9999;

      for (ii = 0; ii < Nimage; ii++)                                      //  find closest image center to mouse
      {
         px0 = image[ii].px0;                                              //  image position and rotation
         py0 = image[ii].py0;                                              //    in layout image space
         sinT = image[ii].sinT;
         cosT = image[ii].cosT;
         ww2 = image[ii].ww2;                                              //  image size at scale
         hh2 = image[ii].hh2;
         px1 = (mx-px0) * cosT + (my-py0) * sinT;                          //  image pixel for layout position
         py1 = (my-py0) * cosT - (mx-px0) * sinT;
         if (px1 < 0 || px1 > ww2-1) continue;                             //  not within image
         if (py1 < 0 || py1 > hh2-1) continue;
         pxlo = image[ii].pxlo;                                            //  image extent
         pxhi = image[ii].pxhi;
         pylo = image[ii].pylo;
         pyhi = image[ii].pyhi;
         cenx = (pxlo + pxhi) / 2;                                         //  image center
         ceny = (pylo + pyhi) / 2;
         distx = mx - cenx;
         disty = my - ceny;
         dist = sqrtf(distx * distx + disty * disty);                      //  mouse/center distance
         if (dist < mindist) {
            mindist = dist;
            Mii = ii;                                                      //  save closest image center
         }
      }
      
      if (Mii < 0) {                                                       //  no image clicked
         select("",-1);                                                    //  no selected image
         draw_mousearc(0,0,0,0,1);                                         //  no mouse circle
         gdk_window_set_cursor(gdkwin,0);
         return;
      }

      ii = Mii; 
      select("image",ii);                                                  //  set selected image
      flashimage();                                                        //  flash selected image
      setlayoutupdatearea();                                               //  set layout update area
      E3update();                                                          //  update layout composite image
      return;
   }

   if (! strmatch(focus,"image")) {                                        //  if no current image,
      draw_mousearc(0,0,0,0,1);                                            //    no mouse circle
      gdk_window_set_cursor(gdkwin,0);
      return;
   }

   ii = focusii;                                                           //  current image
   px0 = image[ii].px0;                                                    //  image position and rotation
   py0 = image[ii].py0;                                                    //    in layout image space
   sinT = image[ii].sinT;
   cosT = image[ii].cosT;
   ww1 = image[ii].ww1;                                                    //  image size at 1x
   hh1 = image[ii].hh1;
   ww2 = image[ii].ww2;                                                    //  image size at scale
   hh2 = image[ii].hh2;
   px1 = (mx-px0) * cosT + (my-py0) * sinT;                                //  mouse position within image
   py1 = (my-py0) * cosT - (mx-px0) * sinT;

   if (px1+mrad < 0 || px1-mrad > ww2 ||                                   //  entire circle off image
       py1+mrad < 0 || py1-mrad > hh2) {
      draw_mousearc(0,0,0,0,1);                                            //  no mouse circle
      gdk_window_set_cursor(gdkwin,0);
      return;
   }

   draw_mousearc(mx,my,mrect,mrect,0);                                     //  mouse within image, draw mouse circle
   gdk_window_set_cursor(gdkwin,dragcursor);
   zmainloop();                                                            //  keep mouse circle visible          v.14.06
   
   if (! Mxdrag && ! Mydrag) return;                                       //  no drag underway

   Mxdrag = Mydrag = 0;                                                    //  stop main window drag after return

   if (image[ii].Vtransp == 0) {                                           //  make transparency map if not already
      cc = ww1 * hh1;                                                      //  transparency map is 1x scale
      image[ii].Vtransp = (uint8 *) zmalloc(cc);                           //    and one uint8 per pixel
      memset(image[ii].Vtransp,0,cc);                                      //  initial values = 0 transparency
      image[ii].Vtranspcc = cc;
   }

   qx = px1 / image[ii].scale;                                             //  mouse position in 1x image
   qy = py1 / image[ii].scale;

   mrad = mrad / image[ii].scale;                                          //  mouse radius at 1x scale

   for (dy = -mrad; dy <= mrad; dy++)                                      //  loop pixels within mouse circle
   for (dx = -mrad; dx <= mrad; dx++)
   {
      dist = sqrtf(dx*dx + dy*dy);                                         //  distance from mouse center
      if (dist >= mrad) continue;                                          //  outside mouse circle
      
      if (Mgradual) {                                                      //  gradual paint transparency
         mtransp = Mpowcen * (mrad - dist) / mrad                          //  mouse power at distance from center
                 + Mpowedge * dist / mrad;                                 //    Mpowcen ... Mpowedge  scale 0-100
         if (drandz() < 0.005) continue;                                   //  thin active points                 v.15.02
      }
      else mtransp = 255;                                                  //  instant paint transparency
      
      rx = qx + dx;                                                        //  1x pixel position
      ry = qy + dy;

      if (rx < 0 || rx > ww1-1) continue;                                  //  outside 1x image
      if (ry < 0 || ry > hh1-1) continue;
         
      jj = ry * ww1 + rx;                                                  //  get corresp. var. transparency
      transp = image[ii].Vtransp[jj];

      if (Mbutton == 1) {                                                  //  left mouse button
         transp += mtransp;                                                //  increase image transparency
         if (transp > 255) transp = 255;                                   //    by mouse power
      }
      else {                                                               //  right button
         transp -= mtransp;                                                //  decrease
         if (transp < 0) transp = 0;
      }
      
      image[ii].Vtransp[jj] = transp;                                      //  adjusted image transparency
   }
   
   updxlo = mx - mrad;                                                     //  update area = mouse circle
   updxhi = mx + mrad;
   updylo = my - mrad;
   updyhi = my + mrad;   

   E3update();                                                             //  update layout composite image
   return;
}


//  select and add one or more new overlay image files

void mashup::add_image()
{
   using namespace mashup;
   
   int            ii, jj, nn, cc, Nfiles;
   int            ww1, hh1, ww2, hh2;
   float          scale;
   char           **selfiles, *file, *infofile, *pp;
   PXM            *pxm1, *pxm1f;

   zdialog_show(zdimage,0);                                                //  hide parent dialog
   selfiles = gallery_getfiles(null);                                      //  select image files from gallery
   m_viewmode(0,"F");                                                      //  restore view mode F
   zdialog_show(zdimage,1);

   if (! selfiles) return;                                                 //  no files selected
   for (nn = 0; selfiles[nn]; nn++);                                       //  count files selected
   Nfiles = nn;

   for (nn = 0; nn < Nfiles; nn++)
   {
      if (Nimage == maxmash) {
         zmessageACK(Mwin,0,ZTX("exceeded %d images"),maxmash);
         break;
      }
      
      file = selfiles[nn];
      pxm1 = PXM_load(file,1);                                             //  load image, ACK errors
      if (! pxm1) {
         zfree(file);
         continue;
      }
      
      PXM_fixblue(pxm1);                                                   //  RGB 0/0/0 pixel >> 0/0/1 for vpixel

      pxm1f = 0;
      if (strstr(file,saved_areas_dirk)) {                                 //  image is from saved select area
         infofile = zstrdup(file,8);
         pp = strstr(infofile,".tiff");
         if (! pp) pp = infofile + strlen(infofile);                       //  load corresp. area info image      v.14.01
         strcpy(pp,".info.tiff");
         pxm1f = PXM_load(infofile,1);
         if (! pxm1f) {
            zfree(file);
            zfree(infofile);
            continue;
         }
      }

      ii = Nimage++;                                                       //  new image
      cc = sizeof(image_t);                                                //  clear image data
      memset(&image[ii],0,cc);

      image[ii].file = file;                                               //  add image file to mashup list
      image[ii].pxm1 = pxm1;
      image[ii].pxm1f = pxm1f;
      scale = 1.0;
      ww1 = pxm1->ww;
      hh1 = pxm1->hh;
      if (scale * ww1 / Lww > 0.3) scale = 0.3 * Lww / ww1;                //  if image > 30% layout, reduce to 30%
      if (scale * hh1 / Lhh > 0.3) scale = 0.3 * Lhh / hh1;
      ww2 = ww1 * scale;
      hh2 = hh1 * scale;
      image[ii].scale = scale;
      image[ii].ww1 = ww1;
      image[ii].hh1 = hh1;
      image[ii].ww2 = ww2;
      image[ii].hh2 = hh2;
      image[ii].pxm2 = PXM_rescale(image[ii].pxm1,ww2,hh2);
      if (pxm1f) 
         image[ii].pxm2f = PXM_rescale(image[ii].pxm1f,ww2,hh2);           //  v.14.01
      image[ii].theta = 0.0;
      image[ii].sinT = 0;
      image[ii].cosT = 1.0;
      image[ii].Btransp = 0.0;
      image[ii].Fblend = 0;                                                //  v.15.02
      image[ii].Lbdist = image[ii].Rbdist = 0;
      image[ii].Tbdist = image[ii].Bbdist = 0;
      image[ii].Vtranspcc = 0;
      image[ii].Vtransp = 0;
      jj = ii;
      while (jj >= 25) jj -= 25;
      image[ii].py0 = (jj / 5) * 0.20 * Lhh;                               //  initial position
      image[ii].px0 = (jj % 5) * 0.20 * Lww;
      select("image",ii);
      setlayoutupdatearea();                                               //  set layout update area
   }
   
   while (nn < Nfiles) zfree(selfiles[nn++]);                              //  free unused files 
   zfree(selfiles);
   
   select("",-1);                                                          //  nothing selected
   Fupdall = 1;
   E3update();                                                             //  update layout composite image
   return;
}


//  remove an overlay image

void mashup::remove_image(int ii)
{
   using namespace mashup;
   
   select("image",ii);
   setlayoutupdatearea();                                                  //  set layout update area

   zfree(image[ii].file);                                                  //  free memory
   PXM_free(image[ii].pxm1);
   PXM_free(image[ii].pxm2);
   PXM_free(image[ii].pxm1f);                                              //  v.14.01
   PXM_free(image[ii].pxm2f);
   if (image[ii].Vtransp) zfree(image[ii].Vtransp);
   
   for (int jj = ii+1; jj < Nimage; jj++)                                  //  pack-down image list
      image[ii] = image[jj];
   Nimage--;                                                               //  image count

   select("",-1);                                                          //  no selected image
   Fupdall = 1;
   E3update();                                                             //  update layout composite image
   return;
}


//  flash a selected image as bright as possible for a short moment

void mashup::flashimage()
{
   int      ii, px, py, ww2, hh2;
   int      xlo, xhi, ylo, yhi;
   float    px0, py0, px1, py1, px2, py2;
   float    sinT, cosT;
   float    *pix3, pix9[3];
   PXM      *pxm2;
   
   if (! strmatch(focus,"image")) return;
   ii = focusii;
   
   xlo = image[ii].pxlo;                                                   //  overlay image extent in layout image
   xhi = image[ii].pxhi;
   ylo = image[ii].pylo;
   yhi = image[ii].pyhi;

   for (py = ylo; py < yhi; py++)
   for (px = xlo; px < xhi; px++)
   {
      pix3 = PXMpix(E3pxm,px,py);                                          //  output image pixel
      px1 = px;
      py1 = py;
      px0 = image[ii].px0;                                                 //  overlay image position and rotation
      py0 = image[ii].py0;                                                 //    in layout image space
      pxm2 = image[ii].pxm2;
      sinT = image[ii].sinT;
      cosT = image[ii].cosT;
      ww2 = image[ii].ww2;                                                 //  image size at scale
      hh2 = image[ii].hh2;
      px2 = (px1-px0) * cosT + (py1-py0) * sinT;                           //  overlay image pixel for layout position
      py2 = (py1-py0) * cosT - (px1-px0) * sinT;
      if (px2 < 0 || px2 > ww2-1) continue;                                //  not within image
      if (py2 < 0 || py2 > hh2-1) continue;
      if (! vpixel(pxm2,px2,py2,pix9)) continue;                           //  overlay image virt. pixel
      pix3[0] = pix9[1];                                                   //  rotate the colors
      pix3[1] = pix9[2];
      pix3[2] = pix9[0];
   }
   
   Fpaint3(xlo,ylo,xhi-xlo,yhi-ylo);                                       //  paint updated region
   zmainloop();                                                            //  added v.14.06
   zsleep(0.05);                                                           //  hold for a moment

   setlayoutupdatearea();                                                  //  set layout update area
   E3update();                                                             //  update layout composite image
   return;
}


//  add text to image or revise existing text

void mashup::text_edit()
{
   using namespace mashup;

   cchar    *intro = ZTX("Enter text, click/drag on image.");

/***
       _____________________________________________________
      |                  Edit Text                          |
      |                                                     |
      |  Enter text, click/drag on image.                   |
      |                                                     |
      |  Text [_________________________________________]   |
      |                                                     |
      |  [font] [FreeSans         ]  size [ 44|v]           |
      |                                                     |
      |            color   transp.   width    angle         |
      |  text     [_____]  [___|v]           [___|v]        |
      |  backing  [_____]  [___|v]                          |
      |  outline  [_____]  [___|v]  [___|v]                 |
      |  shadow   [_____]  [___|v]  [___|v]  [___|v]        |
      |                                                     |
      |  Text File: [Open] [Save]                           |
      |                                                     |
      |                              [Add] [Delete] [Done]  |
      |_____________________________________________________|

***/

   select("",-1);                                                          //  nothing selected

   zdtext = zdialog_new(ZTX("Edit Text"),Mwin,Badd,Bdelete,Bdone,null);
   zdialog *zd = zdtext;

   zdialog_add_widget(zd,"label","intro","dialog",intro,"space=3");
   zdialog_add_widget(zd,"hbox","hbtext","dialog",0,"space=3");
   zdialog_add_widget(zd,"label","labtext","hbtext",ZTX("Text"),"space=5");
   zdialog_add_widget(zd,"frame","frtext","hbtext",0,"expand");
   zdialog_add_widget(zd,"edit","text","frtext","","expand");

   zdialog_add_widget(zd,"hbox","hbfont","dialog",0,"space=8");
   zdialog_add_widget(zd,"button","fontbutt","hbfont",Bfont);
   zdialog_add_widget(zd,"entry","fontname","hbfont","FreeSans","space=2|expand");
   zdialog_add_widget(zd,"label","labsize","hbfont",Bsize,"space=2");
   zdialog_add_widget(zd,"spin","fontsize","hbfont","8|500|1|40");
   
   zdialog_add_widget(zd,"hbox","hbcol","dialog");
   zdialog_add_widget(zd,"vbox","vbcol1","hbcol",0,"homog|space=5");
   zdialog_add_widget(zd,"vbox","vbcol2","hbcol",0,"homog|space=2");
   zdialog_add_widget(zd,"vbox","vbcol3","hbcol",0,"homog|space=2");
   zdialog_add_widget(zd,"vbox","vbcol4","hbcol",0,"homog|space=2");
   zdialog_add_widget(zd,"vbox","vbcol5","hbcol",0,"homog|space=2");

   zdialog_add_widget(zd,"label","space","vbcol1");
   zdialog_add_widget(zd,"label","labtext","vbcol1",ZTX("text"));
   zdialog_add_widget(zd,"label","labback","vbcol1",ZTX("backing"));
   zdialog_add_widget(zd,"label","laboutln","vbcol1",ZTX("outline"));
   zdialog_add_widget(zd,"label","labshadow","vbcol1",ZTX("shadow"));

   zdialog_add_widget(zd,"label","labcol","vbcol2",ZTX("Color"));
   zdialog_add_widget(zd,"colorbutt","fgcolor","vbcol2","0|0|0");
   zdialog_add_widget(zd,"colorbutt","bgcolor","vbcol2","255|255|255");
   zdialog_add_widget(zd,"colorbutt","tocolor","vbcol2","255|0|0");
   zdialog_add_widget(zd,"colorbutt","shcolor","vbcol2","0|255|0");

   zdialog_add_widget(zd,"label","labcol","vbcol3",ZTX("Transparency"));
   zdialog_add_widget(zd,"spin","fgtransp","vbcol3","0|100|1|0");
   zdialog_add_widget(zd,"spin","bgtransp","vbcol3","0|100|1|0");
   zdialog_add_widget(zd,"spin","totransp","vbcol3","0|100|1|0");
   zdialog_add_widget(zd,"spin","shtransp","vbcol3","0|100|1|0");

   zdialog_add_widget(zd,"label","labw","vbcol4",ZTX("Width"));
   zdialog_add_widget(zd,"label","space","vbcol4");
   zdialog_add_widget(zd,"label","space","vbcol4");
   zdialog_add_widget(zd,"spin","towidth","vbcol4","0|30|1|0");
   zdialog_add_widget(zd,"spin","shwidth","vbcol4","0|50|1|0");

   zdialog_add_widget(zd,"label","labw","vbcol5",Bangle);
   zdialog_add_widget(zd,"spin","angle","vbcol5","-180|180|0.1|0");
   zdialog_add_widget(zd,"label","space","vbcol5");
   zdialog_add_widget(zd,"label","space","vbcol5");
   zdialog_add_widget(zd,"spin","shangle","vbcol5","-180|180|1|0");

   zdialog_add_widget(zd,"hbox","hbaf","dialog",0,"space=10");
   zdialog_add_widget(zd,"label","labbg","hbaf",ZTX("Text File:"),"space=3");
   zdialog_add_widget(zd,"button","loadtext","hbaf",Bopen,"space=5");
   zdialog_add_widget(zd,"button","savetext","hbaf",Bsave,"space=5");

   zdialog_add_ttip(zd,Badd,ZTX("add entered text to layout"));            //  v.14.08

   zdialog_restore_inputs(zd);                                             //  restore prior inputs
   zdialog_run(zd,text_dialog_event,"save");                               //  run dialog, parallel
   zdialog_wait(zd);
   zdialog_free(zd);                                                       //  v.14.06

   return;
}


//  dialog event and completion callback function

int mashup::text_dialog_event(zdialog *zd, cchar *event)
{
   using namespace mashup;

   GtkWidget   *font_dialog;
   char        font[80];
   int         ii, size;
   char        *pp;

   if (zd->zstat)
   {
      if (zd->zstat == 1) {                                                //  add
         zd->zstat = 0;
         add_text();                                                       //  zdialog inputs >> new text image
         return 1;
      }
      
      if (zd->zstat == 2) {                                                //  delete
         zd->zstat = 0;
         if (! strmatch(focus,"text")) return 1;
         ii = focusii;
         remove_text(ii);                                                  //  remove selected text image
         return 1;
      }
      
      zdtext = 0;                                                          //  done or [x] kill
      return 1;
   }
   
   if (strmatch(event,"focus")) {                                          //  get mouse ownership
      takeMouse(mousefunc_layout,0);
      return 1;
   }

   if (strmatch(event,"loadtext"))                                         //  load text data from file
      load_text(zdtext);                                                   //    and stuff zdialog fields

   if (strmatch(event,"savetext")) {                                       //  get all zdialog fields
      save_text(zdtext);                                                   //    and save them to a file
      return 1;
   }
   
   if (strmatch(event,"fontbutt"))                                         //  set new font
   {
      zdialog_fetch(zd,"fontname",font,80);
      zdialog_fetch(zd,"fontsize",size);
      snprintf(font,80,"%s %d",font,size);
      font_dialog = gtk_font_chooser_dialog_new(ZTX("select font"),MWIN);
      gtk_font_chooser_set_font(GTK_FONT_CHOOSER(font_dialog),font);
      gtk_dialog_run(GTK_DIALOG(font_dialog));
      pp = gtk_font_chooser_get_font(GTK_FONT_CHOOSER(font_dialog));
      gtk_widget_destroy(font_dialog);

      if (pp) {                                                            //  should have "fontname nn"
         strncpy0(font,pp,80);
         g_free(pp);
         pp = font + strlen(font);
         while (*pp != ' ') pp--;
         if (pp > font) {                                                  //  set both font name and size
            size = atoi(pp);
            if (size < 8) size = 8;
            zdialog_stuff(zd,"fontsize",size);
            *pp = 0;
            zdialog_stuff(zd,"fontname",font);
         }
      }
   }

   if (! strmatch(focus,"text")) return 1;                                 //  selected text image
   ii = focusii;
   
   zdialog_fetch(zd,"text",text[ii].attr.text,1000);                       //  initz. new text and attributes
   zdialog_fetch(zd,"fontname",text[ii].attr.font,80);
   zdialog_fetch(zd,"fontsize",text[ii].attr.size);
   zdialog_fetch(zd,"angle",text[ii].attr.angle);
   zdialog_fetch(zd,"fgcolor",text[ii].attr.color[0],20);
   zdialog_fetch(zd,"bgcolor",text[ii].attr.color[1],20);
   zdialog_fetch(zd,"tocolor",text[ii].attr.color[2],20);
   zdialog_fetch(zd,"shcolor",text[ii].attr.color[3],20);
   zdialog_fetch(zd,"fgtransp",text[ii].attr.transp[0]);
   zdialog_fetch(zd,"bgtransp",text[ii].attr.transp[1]);
   zdialog_fetch(zd,"totransp",text[ii].attr.transp[2]);
   zdialog_fetch(zd,"shtransp",text[ii].attr.transp[3]);
   zdialog_fetch(zd,"towidth",text[ii].attr.towidth);
   zdialog_fetch(zd,"shwidth",text[ii].attr.shwidth);
   zdialog_fetch(zd,"shangle",text[ii].attr.shangle);
   
   gentext(&text[ii].attr);                                                //  generate new text image
   text[ii].ww = text[ii].attr.pxm_text->ww;
   text[ii].hh = text[ii].attr.pxm_text->hh;
   setlayoutupdatearea();                                                  //  set layout update area

   zmainloop();                                                            //  v.14.07
   E3update();                                                             //  update layout composite image

   return 1;
}


//  add new text entry to text[ii] list
//  get text and attributes from zdialog fields

void mashup::add_text()
{
   using namespace mashup;
   
   int            ii, jj;
   zdialog        *zd = zdtext;
   
   if (! zd) return;
   
   if (Ntext == maxmash) {
      zmessageACK(Mwin,0,ZTX("exceeded %d text entries"),maxmash);
      return;
   }

   ii = Ntext++;
   memset(&text[ii],0,sizeof(text_t));                                     //  v.14.10

   zdialog_fetch(zd,"text",text[ii].attr.text,1000);                       //  initz. new text and attributes
   zdialog_fetch(zd,"fontname",text[ii].attr.font,80);
   zdialog_fetch(zd,"fontsize",text[ii].attr.size);
   zdialog_fetch(zd,"angle",text[ii].attr.angle);
   zdialog_fetch(zd,"fgcolor",text[ii].attr.color[0],20);
   zdialog_fetch(zd,"bgcolor",text[ii].attr.color[1],20);
   zdialog_fetch(zd,"tocolor",text[ii].attr.color[2],20);
   zdialog_fetch(zd,"shcolor",text[ii].attr.color[3],20);
   zdialog_fetch(zd,"fgtransp",text[ii].attr.transp[0]);
   zdialog_fetch(zd,"bgtransp",text[ii].attr.transp[1]);
   zdialog_fetch(zd,"totransp",text[ii].attr.transp[2]);
   zdialog_fetch(zd,"shtransp",text[ii].attr.transp[3]);
   zdialog_fetch(zd,"towidth",text[ii].attr.towidth);
   zdialog_fetch(zd,"shwidth",text[ii].attr.shwidth);
   zdialog_fetch(zd,"shangle",text[ii].attr.shangle);

   gentext(&text[ii].attr);                                                //  generate new text image
   text[ii].ww = text[ii].attr.pxm_text->ww;
   text[ii].hh = text[ii].attr.pxm_text->hh;

   jj = ii;
   while (jj >= 25) jj -= 25;
   text[ii].py0 = (jj/5) * 0.20 * Lhh;                                     //  initial position
   text[ii].px0 = (jj%5) * 0.20 * Lww;
   
   select("text",ii);                                                      //  set selected text image
   setlayoutupdatearea();                                                  //  set layout update area
   E3update();                                                             //  update layout composite image

   return;
}


//  remove text entry from text[ii] 

void mashup::remove_text(int ii)
{
   using namespace mashup;
   
   select("text",ii);
   setlayoutupdatearea();                                                  //  set layout update area

   PXM_free(text[ii].attr.pxm_text);
   PXM_free(text[ii].attr.pxm_transp);
   for (int jj = ii+1; jj < Ntext; jj++)                                   //  pack down text list
      text[jj-1] = text[jj];
   Ntext--;                                                                //  text count

   select("",-1);                                                          //  nothing selected
   Fupdall = 1;
   E3update();                                                             //  update layout composite image
   return;
}


//  flash a selected text image as bright as possible for a short moment

void mashup::flashtext()
{
   int         ii, jj, red, green, blue;
   char        savecolor[4][20], flashcolor[20];

   if (! strmatch(focus,"text")) return;
   ii = focusii;

   for (jj = 0; jj < 4; jj++)                                              //  save text colors
      strcpy(savecolor[jj],text[ii].attr.color[jj]);   

   for (jj = 0; jj < 4; jj++) {                                            //  get complimentary colors
      red = 255 - atoi(strField(text[ii].attr.color[jj],'|',1));
      green = 255 - atoi(strField(text[ii].attr.color[jj],'|',2));
      blue = 255 - atoi(strField(text[ii].attr.color[jj],'|',3));
      snprintf(flashcolor,20,"%d|%d|%d",red,green,blue);
      strcpy(text[ii].attr.color[jj],flashcolor);
   }
   
   gentext(&text[ii].attr);                                                //  generate complimentary text image
   
   setlayoutupdatearea();                                                  //  set layout update area
   E3update();                                                             //  update layout composite image
   zsleep(0.05);                                                           //  hold for a moment
   zmainloop();                                                            //  added v.14.06
   
   for (jj = 0; jj < 4; jj++)                                              //  restore text colors
      strcpy(text[ii].attr.color[jj],savecolor[jj]);   

   gentext(&text[ii].attr);                                                //  gemerate normal text image

   E3update();                                                             //  update layout composite image
   return;
}


//  add a line/arrow to image or revise existing line/arrow

void mashup::line_edit()                                                   //  v.14.07
{
   using namespace mashup;

   cchar    *intro = ZTX("Enter line/arrow properties, click/drag on image.");

/***
       _____________________________________________________
      |                  Edit Line/Arrow                    |
      |                                                     |
      |  Enter line/arrow properties, click/drag on image.  |
      |                                                     |
      |  Line length [___|+-]  width [___|+-]               |
      |  Arrow head  [x] left   [x] right                   |
      |                                                     |
      |            color   transp.   width    angle         |
      |  line     [_____]  [___|v]           [___|v]        |
      |  backing  [_____]  [___|v]                          |
      |  outline  [_____]  [___|v]  [___|v]                 |
      |  shadow   [_____]  [___|v]  [___|v]  [___|v]        |
      |                                                     |
      |                              [Add] [Delete] [Done]  |
      |_____________________________________________________|

***/

   select("",-1);                                                          //  nothing selected

   zdline = zdialog_new(ZTX("Edit Line/Arrow"),Mwin,Badd,Bdelete,Bdone,null);
   zdialog *zd = zdline;

   zdialog_add_widget(zd,"label","intro","dialog",intro,"space=3");

   zdialog_add_widget(zd,"hbox","hbline","dialog",0,"space=3");
   zdialog_add_widget(zd,"label","lablength","hbline",ZTX("Line length"),"space=5");
   zdialog_add_widget(zd,"spin","length","hbline","2|9999|1|20");
   zdialog_add_widget(zd,"label","space","hbline",0,"space=10");
   zdialog_add_widget(zd,"label","labwidth","hbline",ZTX("width"),"space=5");
   zdialog_add_widget(zd,"spin","width","hbline","1|99|1|2");

   zdialog_add_widget(zd,"hbox","hbarrow","dialog",0,"space=3");
   zdialog_add_widget(zd,"label","labarrow","hbarrow",ZTX("Arrow head"),"space=5");
   zdialog_add_widget(zd,"check","larrow","hbarrow",Bleft);
   zdialog_add_widget(zd,"label","space","hbarrow",0,"space=10");
   zdialog_add_widget(zd,"check","rarrow","hbarrow",Bright);

   zdialog_add_widget(zd,"hbox","hbcol","dialog");
   zdialog_add_widget(zd,"vbox","vbcol1","hbcol",0,"homog|space=5");
   zdialog_add_widget(zd,"vbox","vbcol2","hbcol",0,"homog|space=2");
   zdialog_add_widget(zd,"vbox","vbcol3","hbcol",0,"homog|space=2");
   zdialog_add_widget(zd,"vbox","vbcol4","hbcol",0,"homog|space=2");
   zdialog_add_widget(zd,"vbox","vbcol5","hbcol",0,"homog|space=2");

   zdialog_add_widget(zd,"label","space","vbcol1");
   zdialog_add_widget(zd,"label","labline","vbcol1",ZTX("line"));
   zdialog_add_widget(zd,"label","labback","vbcol1",ZTX("backing"));
   zdialog_add_widget(zd,"label","laboutln","vbcol1",ZTX("outline"));
   zdialog_add_widget(zd,"label","labshadow","vbcol1",ZTX("shadow"));

   zdialog_add_widget(zd,"label","labcol","vbcol2",ZTX("Color"));
   zdialog_add_widget(zd,"colorbutt","fgcolor","vbcol2","0|0|0");
   zdialog_add_widget(zd,"colorbutt","bgcolor","vbcol2","255|255|255");
   zdialog_add_widget(zd,"colorbutt","tocolor","vbcol2","255|0|0");
   zdialog_add_widget(zd,"colorbutt","shcolor","vbcol2","0|255|0");

   zdialog_add_widget(zd,"label","labcol","vbcol3",ZTX("Transparency"));
   zdialog_add_widget(zd,"spin","fgtransp","vbcol3","0|100|1|0");
   zdialog_add_widget(zd,"spin","bgtransp","vbcol3","0|100|1|0");
   zdialog_add_widget(zd,"spin","totransp","vbcol3","0|100|1|0");
   zdialog_add_widget(zd,"spin","shtransp","vbcol3","0|100|1|0");

   zdialog_add_widget(zd,"label","labw","vbcol4",ZTX("Width"));
   zdialog_add_widget(zd,"label","space","vbcol4");
   zdialog_add_widget(zd,"label","space","vbcol4");
   zdialog_add_widget(zd,"spin","towidth","vbcol4","0|30|1|0");
   zdialog_add_widget(zd,"spin","shwidth","vbcol4","0|50|1|0");

   zdialog_add_widget(zd,"label","labw","vbcol5",Bangle);
   zdialog_add_widget(zd,"spin","angle","vbcol5","-180|180|0.1|0");
   zdialog_add_widget(zd,"label","space","vbcol5");
   zdialog_add_widget(zd,"label","space","vbcol5");
   zdialog_add_widget(zd,"spin","shangle","vbcol5","-180|180|1|0");

   zdialog_add_ttip(zd,Badd,ZTX("add line/arrow to layout"));              //  v.14.08

   zdialog_restore_inputs(zd);                                             //  restore prior inputs
   zdialog_run(zd,line_dialog_event,"save");                               //  run dialog, parallel
   zdialog_wait(zd);
   zdialog_free(zd);                                                       //  v.14.06

   return;
}


//  dialog event and completion callback function

int mashup::line_dialog_event(zdialog *zd, cchar *event)
{
   using namespace mashup;

   int      ii;

   if (zd->zstat)
   {
      if (zd->zstat == 1) {                                                //  add
         zd->zstat = 0;
         add_line();                                                       //  zdialog inputs >> new line/arrow image
         return 1;
      }
      
      if (zd->zstat == 2) {                                                //  delete
         zd->zstat = 0;
         if (! strmatch(focus,"line")) return 1;
         ii = focusii;
         remove_line(ii);                                                  //  remove selected line/arrow image
         return 1;
      }
      
      zdline = 0;                                                          //  done or [x] kill
      return 1;
   }
   
   if (strmatch(event,"focus")) {                                          //  get mouse ownership
      takeMouse(mousefunc_layout,0);
      return 1;
   }

   ii = focusii;

   zdialog_fetch(zd,"length",line[ii].attr.length);
   zdialog_fetch(zd,"width",line[ii].attr.width);
   zdialog_fetch(zd,"larrow",line[ii].attr.larrow);
   zdialog_fetch(zd,"rarrow",line[ii].attr.rarrow);
   zdialog_fetch(zd,"angle",line[ii].attr.angle);
   zdialog_fetch(zd,"fgcolor",line[ii].attr.color[0],20);
   zdialog_fetch(zd,"bgcolor",line[ii].attr.color[1],20);
   zdialog_fetch(zd,"tocolor",line[ii].attr.color[2],20);
   zdialog_fetch(zd,"shcolor",line[ii].attr.color[3],20);
   zdialog_fetch(zd,"fgtransp",line[ii].attr.transp[0]);
   zdialog_fetch(zd,"bgtransp",line[ii].attr.transp[1]);
   zdialog_fetch(zd,"totransp",line[ii].attr.transp[2]);
   zdialog_fetch(zd,"shtransp",line[ii].attr.transp[3]);
   zdialog_fetch(zd,"towidth",line[ii].attr.towidth);
   zdialog_fetch(zd,"shwidth",line[ii].attr.shwidth);
   zdialog_fetch(zd,"shangle",line[ii].attr.shangle);
   
   genline(&line[ii].attr);                                                //  generate new line/arrow image
   line[ii].ww = line[ii].attr.pxm_line->ww;
   line[ii].hh = line[ii].attr.pxm_line->hh;

   setlayoutupdatearea();                                                  //  set layout update area
   zmainloop();                                                            //  v.14.07
   E3update();                                                             //  update layout composite image
   
   return 1;
}


//  add new line/arrow entry to line[ii] list
//  get line attributes from zdialog fields

void mashup::add_line()
{
   using namespace mashup;
   
   int            ii, jj;
   zdialog        *zd = zdline;
   
   if (! zd) return;
   
   if (Nline == maxmash) {
      zmessageACK(Mwin,0,ZTX("exceeded %d line entries"),maxmash);
      return;
   }

   ii = Nline++;
   memset(&line[ii],0,sizeof(line_t));                                     //  v.14.10

   zdialog_fetch(zd,"length",line[ii].attr.length);
   zdialog_fetch(zd,"width",line[ii].attr.width);
   zdialog_fetch(zd,"larrow",line[ii].attr.larrow);
   zdialog_fetch(zd,"rarrow",line[ii].attr.rarrow);
   zdialog_fetch(zd,"angle",line[ii].attr.angle);
   zdialog_fetch(zd,"fgcolor",line[ii].attr.color[0],20);
   zdialog_fetch(zd,"bgcolor",line[ii].attr.color[1],20);
   zdialog_fetch(zd,"tocolor",line[ii].attr.color[2],20);
   zdialog_fetch(zd,"shcolor",line[ii].attr.color[3],20);
   zdialog_fetch(zd,"fgtransp",line[ii].attr.transp[0]);
   zdialog_fetch(zd,"bgtransp",line[ii].attr.transp[1]);
   zdialog_fetch(zd,"totransp",line[ii].attr.transp[2]);
   zdialog_fetch(zd,"shtransp",line[ii].attr.transp[3]);
   zdialog_fetch(zd,"towidth",line[ii].attr.towidth);
   zdialog_fetch(zd,"shwidth",line[ii].attr.shwidth);
   zdialog_fetch(zd,"shangle",line[ii].attr.shangle);
   
   genline(&line[ii].attr);                                                //  generate new line/arrow image
   line[ii].ww = line[ii].attr.pxm_line->ww;
   line[ii].hh = line[ii].attr.pxm_line->hh;

   jj = ii;
   while (jj >= 25) jj -= 25;
   line[ii].py0 = (jj/5) * 0.20 * Lhh;                                     //  initial position
   line[ii].px0 = (jj%5) * 0.20 * Lww;
   
   select("line",ii);                                                      //  set selected line/arrow image
   setlayoutupdatearea();                                                  //  set layout update area
   E3update();                                                             //  update layout composite image

   return;
}


//  remove line/arrow entry from line[ii] 

void mashup::remove_line(int ii)
{
   using namespace mashup;
   
   select("line",ii);
   setlayoutupdatearea();                                                  //  set layout update area

   PXM_free(line[ii].attr.pxm_line);
   PXM_free(line[ii].attr.pxm_transp);
   for (int jj = ii+1; jj < Nline; jj++)                                   //  pack down line list
      line[jj-1] = line[jj];
   Nline--;                                                                //  line/arrow count

   select("",-1);                                                          //  nothing selected
   Fupdall = 1;
   E3update();                                                             //  update layout composite image
   return;
}


//  flash a selected line/arrow image as bright as possible for a short moment

void mashup::flashline()
{
   int         ii, jj, red, green, blue;
   char        savecolor[4][20], flashcolor[20];

   if (! strmatch(focus,"line")) return;
   ii = focusii;
   
   for (jj = 0; jj < 4; jj++)                                              //  save line colors
      strcpy(savecolor[jj],line[ii].attr.color[jj]);   

   for (jj = 0; jj < 4; jj++) {                                            //  get complimentary colors
      red = 255 - atoi(strField(line[ii].attr.color[jj],'|',1));
      green = 255 - atoi(strField(line[ii].attr.color[jj],'|',2));
      blue = 255 - atoi(strField(line[ii].attr.color[jj],'|',3));
      snprintf(flashcolor,20,"%d|%d|%d",red,green,blue);
      strcpy(line[ii].attr.color[jj],flashcolor);
   }
   
   genline(&line[ii].attr);                                                //  generate complimentary image
   
   setlayoutupdatearea();                                                  //  set layout update area
   E3update();                                                             //  update layout composite image
   zsleep(0.05);                                                           //  hold for a moment
   zmainloop();                                                            //  added v.14.06

   for (jj = 0; jj < 4; jj++)                                              //  restore line colors
      strcpy(line[ii].attr.color[jj],savecolor[jj]);   

   genline(&line[ii].attr);                                                //  gemerate normal line image

   E3update();                                                             //  update layout composite image
   return;
}


//  select an image, text image, or line/arrow image - update active dialog
//  which = "image" or "text" or "" to select nothing

void mashup::select(cchar *which, int ii)
{
   using namespace mashup;

   char     *pp;
   float    rad = 180 / PI;
   zdialog  *zd;

   if (strmatch(which,"image") && ii >= 0)
   {
      if (ii >= Nimage) zappcrash("bad image index");
      focus = "image";                                                     //  for KB key drags
      focusii = ii;
      
      zd = zdimage;
      if (! zd) return;                                                    //  no active edit image dialog
      
      pp = strrchr(image[ii].file,'/');                                    //  stuff zdialog fields
      zdialog_stuff(zd,"currfile",pp+1);                                   //    from selected image
      zdialog_stuff(zd,"scale",image[ii].scale);
      zdialog_stuff(zd,"angle",image[ii].theta * rad);
      zdialog_stuff(zd,"Btransp",image[ii].Btransp);
      zdialog_stuff(zd,"Lbdist",image[ii].Lbdist);                         //  v.15.02
      zdialog_stuff(zd,"Rbdist",image[ii].Rbdist);
      zdialog_stuff(zd,"Tbdist",image[ii].Tbdist);
      zdialog_stuff(zd,"Bbdist",image[ii].Bbdist);
      return;
   }

   if (strmatch(which,"text") && ii >= 0)
   {
      if (ii >= Ntext) zappcrash("bad text index");
      focus = "text";                                                      //  for KB key drags
      focusii = ii;
      
      zd = zdtext;
      if (! zd) return;                                                    //  no active edit text dialog

      zdialog_stuff(zd,"text",text[ii].attr.text);                         //  stuff zdialog fields
      zdialog_stuff(zd,"fontname",text[ii].attr.font);                     //    from selected text image
      zdialog_stuff(zd,"fontsize",text[ii].attr.size);
      zdialog_stuff(zd,"angle",text[ii].attr.angle);
      zdialog_stuff(zd,"fgcolor",text[ii].attr.color[0]);
      zdialog_stuff(zd,"bgcolor",text[ii].attr.color[1]);
      zdialog_stuff(zd,"tocolor",text[ii].attr.color[2]);
      zdialog_stuff(zd,"shcolor",text[ii].attr.color[3]);
      zdialog_stuff(zd,"fgtransp",text[ii].attr.transp[0]);
      zdialog_stuff(zd,"bgtransp",text[ii].attr.transp[1]);
      zdialog_stuff(zd,"totransp",text[ii].attr.transp[2]);
      zdialog_stuff(zd,"shtransp",text[ii].attr.transp[3]);
      zdialog_stuff(zd,"towidth",text[ii].attr.towidth);
      zdialog_stuff(zd,"shwidth",text[ii].attr.shwidth);
      zdialog_stuff(zd,"shangle",text[ii].attr.shangle);
      return;
   }

   if (strmatch(which,"line") && ii >= 0)                                  //  v.14.07
   {
      if (ii >= Nline) zappcrash("bad line index");
      focus = "line";                                                      //  for KB key drags
      focusii = ii;
      
      zd = zdline;
      if (! zd) return;                                                    //  no active edit line/arrow dialog

      zdialog_stuff(zd,"length",line[ii].attr.length);
      zdialog_stuff(zd,"width",line[ii].attr.width);
      zdialog_stuff(zd,"larrow",line[ii].attr.larrow);
      zdialog_stuff(zd,"rarrow",line[ii].attr.rarrow);
      zdialog_stuff(zd,"angle",line[ii].attr.angle);
      zdialog_stuff(zd,"fgcolor",line[ii].attr.color[0]);
      zdialog_stuff(zd,"bgcolor",line[ii].attr.color[1]);
      zdialog_stuff(zd,"tocolor",line[ii].attr.color[2]);
      zdialog_stuff(zd,"shcolor",line[ii].attr.color[3]);
      zdialog_stuff(zd,"fgtransp",line[ii].attr.transp[0]);
      zdialog_stuff(zd,"bgtransp",line[ii].attr.transp[1]);
      zdialog_stuff(zd,"totransp",line[ii].attr.transp[2]);
      zdialog_stuff(zd,"shtransp",line[ii].attr.transp[3]);
      zdialog_stuff(zd,"towidth",line[ii].attr.towidth);
      zdialog_stuff(zd,"shwidth",line[ii].attr.shwidth);
      zdialog_stuff(zd,"shangle",line[ii].attr.shangle);
      return;
   }

   focus = "";                                                             //  no selection
   focusii = -1;
   zd = zdimage;
   if (! zd) return;
   zdialog_stuff(zd,"currfile","");                                        //  clear dialog
   zdialog_stuff(zd,"Btransp",0);
   zdialog_stuff(zd,"Lbdist",0);                                           //  v.15.02
   zdialog_stuff(zd,"Rbdist",0);
   zdialog_stuff(zd,"Tbdist",0);
   zdialog_stuff(zd,"Bbdist",0);
   return;
}


//  Set the update area for the layout based on 
//  the current selected image, text, or line/arrow.

void mashup::setlayoutupdatearea()
{
   using namespace mashup;
   
   int         pxlo, pxhi, pylo, pyhi;
   int         ii, px, py, px0, py0;
   int         ww, hh, ww2, hh2;
   float       cosT, sinT;

   if (strmatch(focus,"image"))
   {
      ii = focusii;
      px0 = image[ii].px0;
      py0 = image[ii].py0;
      ww2 = image[ii].ww2;                                                 //  image size at scale
      hh2 = image[ii].hh2;
      cosT = image[ii].cosT;
      sinT = image[ii].sinT;
      
      pxlo = pxhi = px0;                                                   //  NW corner
      pylo = pyhi = py0;

      px = px0 + ww2 * cosT;                                               //  NE
      if (px < pxlo) pxlo = px;
      if (px > pxhi) pxhi = px;
      py = py0 + ww2 * sinT;
      if (py < pylo) pylo = py;
      if (py > pyhi) pyhi = py;

      px = px0 + ww2 * cosT - hh2 * sinT;                                  //  SE
      if (px < pxlo) pxlo = px;
      if (px > pxhi) pxhi = px;
      py = py0 + hh2 * cosT + ww2 * sinT;
      if (py < pylo) pylo = py;
      if (py > pyhi) pyhi = py;

      px = px0 - hh2 * sinT;                                               //  SW
      if (px < pxlo) pxlo = px;
      if (px > pxhi) pxhi = px;
      py = py0 + hh2 * cosT;
      if (py < pylo) pylo = py;
      if (py > pyhi) pyhi = py;

      if (pxlo < 0) pxlo = 0;
      if (pxhi > Lww) pxhi = Lww;
      if (pylo < 0) pylo = 0;
      if (pyhi > Lhh) pyhi = Lhh;
      
      image[ii].pxlo = pxlo;
      image[ii].pxhi = pxhi;
      image[ii].pylo = pylo;
      image[ii].pyhi = pyhi;
      
      updxlo = pxlo;                                                       //  set update region
      updxhi = pxhi;
      updylo = pylo;
      updyhi = pyhi;
   }
   
   else if (strmatch(focus,"text"))
   {
      ii = focusii;
      px0 = text[ii].px0;
      py0 = text[ii].py0;
      ww = text[ii].ww;
      hh = text[ii].hh;
      
      updxlo = text[ii].pxlo = px0;                                        //  set update region
      updxhi = text[ii].pxhi = px0 + ww;
      updylo = text[ii].pylo = py0;
      updyhi = text[ii].pyhi = py0 + hh;
   }
   
   else if (strmatch(focus,"line"))                                        //  v.14.07
   {
      ii = focusii;
      px0 = line[ii].px0;
      py0 = line[ii].py0;
      ww = line[ii].ww;
      hh = line[ii].hh;
      
      updxlo = line[ii].pxlo = px0;                                        //  set update region
      updxhi = line[ii].pxhi = px0 + ww;
      updylo = line[ii].pylo = py0;
      updyhi = line[ii].pyhi = py0 + hh;
   }
   
   else {
      printz("setlayoutupdatearea() no focus \n");
      return;
   }
   
   return;
}


//  mouse function - called for mouse events inside layout image
//  move and resize the overlay images, text images, line/arrow images

void mashup::mousefunc_layout()
{
   using namespace mashup;

   static int  mdx0 = 0, mdy0 = 0, mdx1, mdy1;
   static int  Mcen, Mcor, Tcen, Tcor, Lcen, Lcor;                         //  mouse at image/text/line center/corner
   static int  iiMcen, iiTcen, iiLcen, iiMcor, iiTcor, iiLcor;             //  selected image/text/line center/corner
   static int  Tcorx, Tcory;
   int         cenx, ceny;
   int         minMcen, minMcor, minTcen, minTcor, minLcen, minLcor;
   int         ii, Fflash, Fmove, Fnew;
   int         mx, my, mx0, my0;
   int         ww, hh, ww1, hh1, ww2, hh2; 
   int         pxlo, pxhi, pylo, pyhi;
   int         length;
   float       dist, distx, disty, Cdist, Mdist, Mdist0;
   float       expand, contract;
   float       scale, thresh, size;
   float       px0, py0, px1, py1, px2, py2, px3, py3;
   float       tx0, ty0, tx1, ty1, tx2, ty2, tx3, ty3;
   float       Vcx, Vcy, Vmx, Vmy;
   float       sinT, cosT;
   float       leng2, amx, amy, d1, d2, angle, rad;
   
   if (LMclick) {                                                          //  left click
      LMclick = 0;
      mx0 = mx = Mxclick;                                                  //  mouse position
      my0 = my = Myclick;
      Fnew = 1;                                                            //  new image or text selected
      Fmove = 0;
      Fflash = 1;
   }

   else if (Mxdrag || Mydrag)                                              //  drag underway
   {
      Fnew = 0;                                                            //  assume same image/text as before
      Fmove = 1;                                                           //    is being moved
      Fflash = 0;

      if (Mxdown != mdx0 || Mydown != mdy0) {                              //  new drag was initiated
         mdx0 = mdx1 = Mxdown;
         mdy0 = mdy1 = Mydown;
         Fnew = 1;                                                         //  new image or text selected
      }  

      mx0 = mdx1;                                                          //  drag start
      my0 = mdy1;
      mx = Mxdrag;                                                         //  drag position
      my = Mydrag;
      mdx1 = mx;                                                           //  next drag start
      mdy1 = my;
      if (mx == mx0 && my == my0) Fmove = 0;                               //  no mouse movement

      Mxdrag = Mydrag = 0;                                                 //  stop main window drag after return
   }

   else return;                                                            //  ignore simple mouse movement
   
   thresh = 50 / Mscale;                                                   //  corner match threshhold, pixels

   if (Fnew)                                                               //  search all images and text images
   {
      minMcen = 9999;                                                      //  find closest image center to mouse

      for (ii = 0; ii < Nimage; ii++)
      {
         px0 = image[ii].px0;                                              //  image position and rotation
         py0 = image[ii].py0;                                              //    in layout image space
         sinT = image[ii].sinT;
         cosT = image[ii].cosT;
         ww2 = image[ii].ww2;                                              //  image size at scale
         hh2 = image[ii].hh2;
         px2 = (mx-px0) * cosT + (my-py0) * sinT;                          //  image pixel for layout position
         py2 = (my-py0) * cosT - (mx-px0) * sinT;
         if (px2 < 0 || px2 > ww2-1) continue;                             //  mouse not within image
         if (py2 < 0 || py2 > hh2-1) continue;
         pxlo = image[ii].pxlo;                                            //  image extent
         pxhi = image[ii].pxhi;
         pylo = image[ii].pylo;
         pyhi = image[ii].pyhi;
         cenx = (pxlo + pxhi) / 2;                                         //  image center
         ceny = (pylo + pyhi) / 2;
         distx = mx - cenx;
         disty = my - ceny;
         dist = sqrtf(distx * distx + disty * disty);                      //  mouse/center distance
         if (dist < minMcen) {
            minMcen = dist;
            iiMcen = ii;                                                   //  save closest image center
         }
      }

      minMcor = 9999;                                                      //  find closest image corner to mouse

      for (ii = 0; ii < Nimage; ii++)
      {
         px0 = image[ii].px0;                                              //  image position and rotation
         py0 = image[ii].py0;                                              //    in layout image space
         sinT = image[ii].sinT;
         cosT = image[ii].cosT;
         ww2 = image[ii].ww2;                                              //  image size at scale
         hh2 = image[ii].hh2;
         px2 = (mx-px0) * cosT + (my-py0) * sinT;                          //  image pixel for layout position
         py2 = (my-py0) * cosT - (mx-px0) * sinT;
         if (px2 < 0 || px2 > ww2-1) continue;                             //  mouse not within image
         if (py2 < 0 || py2 > hh2-1) continue;

         px0 = image[ii].px0;                                              //  NW corner
         py0 = image[ii].py0;
         distx = mx - px0;
         disty = my - py0;
         dist = sqrtf(distx * distx + disty * disty);                      //  mouse/corner distance
         if (dist < minMcor && dist < thresh) {
            minMcor = dist;
            iiMcor = ii;                                                   //  save closest image corner
         }

         px1 = px0 + ww2 * cosT;                                           //  NE
         py1 = py0 + ww2 * sinT;  
         distx = mx - px1;
         disty = my - py1;
         dist = sqrtf(distx * distx + disty * disty);
         if (dist < minMcor && dist < thresh) {
            minMcor = dist;
            iiMcor = ii;
         }

         px2 = px0 + ww2 * cosT - hh2 * sinT;                              //  SE
         py2 = py0 + hh2 * cosT + ww2 * sinT;
         distx = mx - px2;
         disty = my - py2;
         dist = sqrtf(distx * distx + disty * disty);
         if (dist < minMcor && dist < thresh) {
            minMcor = dist;
            iiMcor = ii;
         }

         px3 = px0 - hh2 * sinT;                                           //  SW
         py3 = py0 + hh2 * cosT;
         distx = mx - px3;
         disty = my - py3;
         dist = sqrtf(distx * distx + disty * disty);
         if (dist < minMcor && dist < thresh) {
            minMcor = dist;
            iiMcor = ii;
         }
      }

      minTcen = 9999;                                                      //  find closest text image center to mouse

      for (ii = 0; ii < Ntext; ii++)
      {
         pxlo = text[ii].pxlo;                                             //  image extent
         pxhi = text[ii].pxhi;
         pylo = text[ii].pylo;
         pyhi = text[ii].pyhi;
         if (mx < pxlo || mx > pxhi) continue;                             //  mouse not within text image
         if (my < pylo || my > pyhi) continue;

         cenx = (pxlo + pxhi) / 2;                                         //  text image center
         ceny = (pylo + pyhi) / 2;
         distx = mx - cenx;
         disty = my - ceny;
         dist = sqrtf(distx * distx + disty * disty);                      //  mouse/center distance
         if (dist < minTcen) {
            minTcen = dist;
            iiTcen = ii;
         }
      }

      minTcor = 9999;                                                      //  find closest text image corner to mouse
      Tcorx = Tcory = 0;

      for (ii = 0; ii < Ntext; ii++)
      {
         pxlo = text[ii].pxlo;                                             //  text image extent
         pxhi = text[ii].pxhi;
         pylo = text[ii].pylo;
         pyhi = text[ii].pyhi;
         if (mx < pxlo || mx > pxhi) continue;                             //  mouse not within text image
         if (my < pylo || my > pyhi) continue;

         ww = text[ii].attr.tww;                                           //  unrotated text rectangle size
         hh = text[ii].attr.thh;
         sinT = text[ii].attr.sinT;                                        //  angle of text
         cosT = text[ii].attr.cosT;
         px0 = text[ii].px0;                                               //  NW corner of enclosing rectangle
         py0 = text[ii].py0;
         
         if (text[ii].attr.angle > 0) {
            tx0 = px0 + hh * sinT;                                         //  NW corner of text rectangle
            ty0 = py0;
            tx1 = px0 + ww * cosT + hh * sinT;                             //  NE
            ty1 = py0 + ww * sinT;
            tx2 = px0 + ww * cosT;                                         //  SE
            ty2 = py0 + ww * sinT + hh * cosT;
            tx3 = px0;                                                     //  SW
            ty3 = py0 + hh * cosT;
         }
         else {
            sinT = -sinT;
            tx0 = px0;                                                     //  NW
            ty0 = py0 + ww * sinT;
            tx1 = px0 + ww * cosT;                                         //  NE
            ty1 = py0;
            tx2 = px0 + ww * cosT + hh * sinT;                             //  SE
            ty2 = py0 + hh * cosT;
            tx3 = px0 + hh * sinT;                                         //  SW
            ty3 = py0 + ww * sinT + hh * cosT;
         }
         
         distx = mx - tx0;                                                 //  NW corner
         disty = my - ty0;
         dist = sqrtf(distx * distx + disty * disty);                      //  mouse/corner distance
         if (dist < minTcor && dist < thresh) {
            minTcor = dist;
            iiTcor = ii;
            Tcorx = tx0;                                                   //  save closest text image corner
            Tcory = ty0;
         }

         distx = mx - tx1;                                                 //  NE
         disty = my - ty1;
         dist = sqrtf(distx * distx + disty * disty);
         if (dist < minTcor && dist < thresh) {
            minTcor = dist;
            iiTcor = ii;
            Tcorx = tx1;
            Tcory = ty1;
         }

         distx = mx - tx2;                                                 //  SE
         disty = my - ty2;
         dist = sqrtf(distx * distx + disty * disty);
         if (dist < minTcor && dist < thresh) {
            minTcor = dist;
            iiTcor = ii;
            Tcorx = tx2;
            Tcory = ty2;
         }

         distx = mx - tx3;                                                 //  SW
         disty = my - ty3;
         dist = sqrtf(distx * distx + disty * disty);
         if (dist < minTcor && dist < thresh) {
            minTcor = dist;
            iiTcor = ii;
            Tcorx = tx3;
            Tcory = ty3;
         }
      }
      
      minLcen = 9999;                                                      //  find closest line/arrow image      v.14.07
                                                                           //     center to mouse position
      for (ii = 0; ii < Nline; ii++)
      {
         pxlo = line[ii].pxlo;                                             //  image extent
         pxhi = line[ii].pxhi;
         pylo = line[ii].pylo;
         pyhi = line[ii].pyhi;
         if (mx < pxlo || mx > pxhi) continue;                             //  mouse not within line image
         if (my < pylo || my > pyhi) continue;

         cenx = (pxlo + pxhi) / 2;                                         //  line image center
         ceny = (pylo + pyhi) / 2;
         distx = mx - cenx;
         disty = my - ceny;
         dist = sqrtf(distx * distx + disty * disty);                      //  mouse/center distance
         if (dist < minLcen) {
            minLcen = dist;
            iiLcen = ii;
         }
      }

      minLcor = 9999;                                                      //  find closest line end point to mouse

      for (ii = 0; ii < Nline; ii++)                                       //  v.14.10
      {
         pxlo = line[ii].pxlo;                                             //  line image extent
         pxhi = line[ii].pxhi;
         pylo = line[ii].pylo;
         pyhi = line[ii].pyhi;
         if (mx < pxlo || mx > pxhi) continue;                             //  mouse not within line image
         if (my < pylo || my > pyhi) continue;

         angle = line[ii].attr.angle;
         rad = -angle / 57.296;   

         length = line[ii].attr.length;
         leng2 = length / 2.0;
         
         ww2 = line[ii].attr.pxm_line->ww;                                 //  line image
         hh2 = line[ii].attr.pxm_line->hh;

         amx = ww2 / 2.0;                                                  //  line midpoint within line image
         amy = hh2 / 2.0;

         px1 = amx - leng2 * cosf(rad) + 0.5;                              //  line end points
         py1 = amy + leng2 * sinf(rad) + 0.5;
         px2 = amx + leng2 * cosf(rad) + 0.5;
         py2 = amy - leng2 * sinf(rad) + 0.5;
         
         d1 = (mx-px1-pxlo) * (mx-px1-pxlo) + (my-py1-pylo) * (my-py1-pylo);
         d2 = (mx-px2-pxlo) * (mx-px2-pxlo) + (my-py2-pylo) * (my-py2-pylo);

         d1 = sqrtf(d1);                                                   //  mouse - end point distance
         d2 = sqrtf(d2);

         if (d1 < minLcor && d1 < thresh) {
            minLcor = d1;
            iiLcor = ii;
         }

         if (d2 < minLcor && d2 < thresh) {
            minLcor = d2;
            iiLcor = ii;
         }
      }

      Mcen = Mcor = Tcen = Tcor = Lcen = Lcor = 0;                         //  mark where mouse is acting:
      
      if (Fmove)                                                           //  recognize drags from anywhere
      {
         if (minMcen < minMcor && minMcen < minTcen && minMcen < minTcor && minMcen < minLcen && minMcen < minLcor)
            Mcen = 1;
         if (minMcor < minMcen && minMcor < minTcen && minMcor < minTcor && minMcor < minLcen && minMcor < minLcor)
            Mcor = 1;
         if (minTcen < minMcen && minTcen < minMcor && minTcen < minTcor && minTcen < minLcen && minTcen < minLcor)
            Tcen = 1;
         if (minTcor < minMcen && minTcor < minMcor && minTcor < minTcen && minTcor < minLcen && minTcor < minLcor)
            Tcor = 1;
         if (minLcen < minMcen && minLcen < minMcor && minLcen < minTcen && minLcen < minTcor && minLcen < minLcor)
            Lcen = 1;
         if (minLcor < minMcen && minLcor < minMcor && minLcor < minTcen && minLcor < minTcor && minLcor < minLcen)
            Lcor = 1;
      }
      else                                                                 //  recognize clicks near center only
      {
         if (minMcen < minTcen && minMcen < minLcen)
            Mcen = 1;
         if (minTcen < minMcen && minTcen < minLcen)
            Tcen = 1;
         if (minLcen < minMcen && minLcen < minTcen)
            Lcen = 1;
      }
   }
   
   if (Mcen) { ii = iiMcen; select("image",ii); }                          //  select the corresp. image, text or line
   if (Mcor) { ii = iiMcor; select("image",ii); }
   if (Tcen) { ii = iiTcen; select("text",ii); }
   if (Tcor) { ii = iiTcor; select("text",ii); }
   if (Lcen) { ii = iiLcen; select("line",ii); }
   if (Lcor) { ii = iiLcor; select("line",ii); }

   if (Fflash && Mcen) flashimage();                                       //  if click on image, flash it
   if (Fflash && Tcen) flashtext();                                        //  if click on text, flash it
   if (Fflash && Lcen) flashline();                                        //  if click on line/arrow, flash it

   if (Fmove)
   {
      if (Mcen)                                                            //  move image
      {
         ii = iiMcen;
         select("image",ii);
         image[ii].px0 += (mx - mx0);                                      //  move selected image
         image[ii].py0 += (my - my0);                                      //    by mouse drag amount
      }
      
      if (Mcor)                                                            //  resize image by dragging a corner
      {
         ii = iiMcor;
         select("image",ii);

         pxlo = image[ii].pxlo;                                            //  image extent      bugfix
         pxhi = image[ii].pxhi;
         pylo = image[ii].pylo;
         pyhi = image[ii].pyhi;
         cenx = (pxlo + pxhi) / 2;                                         //  image center
         ceny = (pylo + pyhi) / 2;

         distx = mx0 - cenx;
         disty = my0 - ceny;
         Mdist0 = sqrtf(distx * distx + disty * disty);                    //  center - mouse drag start
         distx = mx - cenx;
         disty = my - ceny;
         Mdist = sqrtf(distx * distx + disty * disty);                     //  center - mouse drag end

         scale = image[ii].scale * 0.5 * (Mdist + Mdist0) / Mdist0;        //  image scale change
         image[ii].scale = scale;
         
         ww1 = image[ii].ww1;                                              //  image dimensions, 1x
         hh1 = image[ii].hh1;
         ww2 = ww1 * scale;                                                //  image dimensions at scale
         hh2 = hh1 * scale;
         PXM_free(image[ii].pxm2);
         image[ii].pxm2 = PXM_rescale(image[ii].pxm1,ww2,hh2);             //  rescale image
         image[ii].ww2 = ww2;
         image[ii].hh2 = hh2;
         PXM_free(image[ii].pxm2f);
         if (image[ii].pxm1f)                                              //  rescale info image                 v.14.02
            image[ii].pxm2f = PXM_rescale(image[ii].pxm1f,ww2,hh2);
      }

      if (Tcen)                                                            //  move text image
      {
         ii = iiTcen;
         select("text",ii);
         text[ii].px0 += (mx - mx0);                                       //  move selected text image
         text[ii].py0 += (my - my0);                                       //    by mouse drag amount
      }

      if (Tcor)                                                            //  resize text image
      {
         ii = iiTcor;
         select("text",ii);

         pxlo = text[ii].pxlo;                                             //  text image extent         bugfix
         pxhi = text[ii].pxhi;
         pylo = text[ii].pylo;
         pyhi = text[ii].pyhi;
         cenx = (pxlo + pxhi) / 2;                                         //  text image center
         ceny = (pylo + pyhi) / 2;
         distx = Tcorx - cenx;
         disty = Tcory - ceny;
         Cdist = sqrtf(distx * distx + disty * disty);
         Vcx = distx / Cdist;                                              //  unit vector, text image center to corner
         Vcy = disty / Cdist;

         distx = mx - mx0;
         disty = my - my0;
         Mdist = sqrtf(distx * distx + disty * disty);
         Vmx = distx / Mdist;                                              //  unit vector, mouse movement
         Vmy = disty / Mdist;
         
         expand   = fabsf(Vcx + Vmx) + fabsf(Vcy + Vmy);                   //  mouse pull-out amount
         contract = fabsf(Vcx - Vmx) + fabsf(Vcy - Vmy);                   //  push-in amount
         size = expand - contract;                                         //  change in text size
         if (size > 0.5) text[ii].attr.size++;
         if (size < -0.5) text[ii].attr.size--;

         gentext(&text[ii].attr);                                          //  generate new text image
         text[ii].ww = text[ii].attr.pxm_text->ww;
         text[ii].hh = text[ii].attr.pxm_text->hh;
      }

      if (Lcen)                                                            //  move line/arrow image              v.14.07
      {
         ii = iiLcen;
         select("line",ii);
         line[ii].px0 += (mx - mx0);                                       //  move selected line image
         line[ii].py0 += (my - my0);                                       //    by mouse drag amount
      }

      if (Lcor)                                                            //  drag line end point                v.14.10
      {
         ii = iiLcor;
         select("line",ii);
         
         angle = line[ii].attr.angle;
         rad = -angle / 57.296;   

         length = line[ii].attr.length;
         leng2 = length / 2.0;
         
         ww2 = line[ii].attr.pxm_line->ww;                                 //  line image
         hh2 = line[ii].attr.pxm_line->hh;

         amx = ww2 / 2.0;                                                  //  line midpoint within line image
         amy = hh2 / 2.0;

         px1 = amx - leng2 * cosf(rad) + 0.5;                              //  line end points
         py1 = amy + leng2 * sinf(rad) + 0.5;
         px2 = amx + leng2 * cosf(rad) + 0.5;
         py2 = amy - leng2 * sinf(rad) + 0.5;
         
         pxlo = line[ii].pxlo;
         pylo = line[ii].pylo;

         d1 = (mx-px1-pxlo) * (mx-px1-pxlo) + (my-py1-pylo) * (my-py1-pylo);
         d2 = (mx-px2-pxlo) * (mx-px2-pxlo) + (my-py2-pylo) * (my-py2-pylo);

         d1 = sqrtf(d1);                                                   //  mouse - end point distance
         d2 = sqrtf(d2);
         
         if (d1 < d2) {                                                    //  move px1/py1 end to mouse
            px2 += pxlo;
            py2 += pylo;
            px1 = mx;
            py1 = my;
            length = d2 + 0.5;
            rad = asinf((py1-py2) / d2);
            angle = -57.296 * rad;
            if (mx > px2) angle = -180 - angle;
         }
         
         if (d2 < d1) {                                                    //  move px2/py2 end to mouse
            px1 += pxlo;
            py1 += pylo;
            px2 = mx;
            py2 = my;
            length = d1 + 0.5;
            rad = asinf((py1-py2) / d1);
            angle = -57.296 * rad;
            if (mx < px1) angle = -180 - angle;
         }
         
         if (angle < -180) angle += 360;
         if (angle > 180) angle -= 360;
         line[ii].attr.angle = angle;
         line[ii].attr.length = length;
         genline(&line[ii].attr);

         line[ii].ww = line[ii].attr.pxm_line->ww;
         line[ii].hh = line[ii].attr.pxm_line->hh;
         ww2 = line[ii].attr.pxm_line->ww;
         hh2 = line[ii].attr.pxm_line->hh;
         amx = (px1 + px2) / 2.0;
         amy = (py1 + py2) / 2.0;
         pxlo = amx - ww2 / 2.0;
         pylo = amy - hh2 / 2.0;
         line[ii].px0 = pxlo;
         line[ii].py0 = pylo;
      }
   }

   setlayoutupdatearea();
   E3update();                                                             //  update layout composite image
   return;
}


//  Keyboard function
//  KB arrow keys act like mouse drag of 1 pixel for current image

void mashup::KBfunc(int key)
{
   using namespace mashup;
   
   int      ii, xstep, ystep;
   
   xstep = ystep = 0;
   if (key == GDK_KEY_Left) xstep = -1;
   if (key == GDK_KEY_Right) xstep = +1;
   if (key == GDK_KEY_Up) ystep = -1;
   if (key == GDK_KEY_Down) ystep = +1;
   
   if (strmatch(focus,"image"))                                            //  choose last selected image
   {
      ii = focusii;
      image[ii].px0 += xstep;                                              //  move image 1 pixel
      image[ii].py0 += ystep;
   }

   else if (strmatch(focus,"text"))                                        //  last selected text image
   {
      ii = focusii;
      text[ii].px0 += xstep;                                               //  move selected text image 1 pixel
      text[ii].py0 += ystep;
   }
   
   else if (strmatch(focus,"line"))                                        //  last selected line/arrow image     v.14.07
   {
      ii = focusii;
      line[ii].px0 += xstep;                                               //  move selected line image 1 pixel
      line[ii].py0 += ystep;
   }
   
   else return;
   
   setlayoutupdatearea();                                                  //  set layout update area
   E3update();                                                             //  update layout composite image
   return;
}


//  update layout image E3 from E1 + overlay image data
//  paint main window

void mashup::E3update()
{
   using namespace mashup;
   
   int            nupdxlo, nupdxhi, nupdylo, nupdyhi;
   static int     pupdxlo, pupdxhi, pupdylo, pupdyhi;                      //  prior image update region
   static cchar   *pfocus = "x";
   static int     pfocusii = -2;                                           //  no prior selected image or text
   
   if (! E3pxm) return;                                                    //  done or canceled

   if (! strmatch(focus,pfocus) || focusii != pfocusii)                    //  focus image/text/line has changed
      Fupdall = 1;                                                         //  update entire layout

   pfocus = focus;                                                         //  remember focus image/text/line
   pfocusii = focusii;

   if (updxlo < 0) updxlo = 0;                                             //  keep within layout image
   if (updxhi > Lww) updxhi = Lww;
   if (updylo < 0) updylo = 0;
   if (updyhi > Lhh) updyhi = Lhh;

   nupdxlo = updxlo;                                                       //  save update region
   nupdxhi = updxhi;
   nupdylo = updylo;
   nupdyhi = updyhi;

   if (! Fupdall) {
      if (pupdxlo < updxlo) updxlo = pupdxlo;                              //  expand update region to
      if (pupdxhi > updxhi) updxhi = pupdxhi;                              //    include prior update region
      if (pupdylo < updylo) updylo = pupdylo;
      if (pupdyhi > updyhi) updyhi = pupdyhi;
   }

   pupdxlo = nupdxlo;                                                      //  save update region
   pupdxhi = nupdxhi;   
   pupdylo = nupdylo;
   pupdyhi = nupdyhi;

   if (Fupdall) {                                                          //  update entire layout
      updxlo = updylo = 0;
      updxhi = Lww;
      updyhi = Lhh;
   }
   
   signal_thread();                                                        //  trigger image update thread
   wait_thread_idle();
   
   if (Fupdall) Fpaintnow();                                               //  paint entire layout
   else Fpaint3(updxlo,updylo,updxhi-updxlo,updyhi-updylo);                //  paint only updated region
   Fupdall = 0;

   CEF->Fmods++;                                                           //  image is modified
   CEF->Fsaved = 0;
   return;
}


void * mashup::E3update_thread(void *)                                     //  main thread
{
   using namespace mashup;

   while (true)
   {
      thread_idle_loop();                                                  //  wait for work or exit request

      for (int ii = 0; ii < NWT; ii++)                                     //  start worker threads for images
         start_wthread(E3update_wthreadimage,&Nval[ii]);
      wait_wthreads();                                                     //  wait for completion

      for (int ii = 0; ii < NWT; ii++)                                     //  start worker threads for text images
         start_wthread(E3update_wthreadtext,&Nval[ii]);
      wait_wthreads();                                                     //  wait for completion

      for (int ii = 0; ii < NWT; ii++)                                     //  start worker threads for line/arrow images
         start_wthread(E3update_wthreadline,&Nval[ii]);
      wait_wthreads();                                                     //  wait for completion
   }

   return 0;                                                               //  not executed, stop warning
}


void * mashup::E3update_wthreadimage(void *arg)                            //  worker thread for images     revised v.14.07
{
   using namespace mashup;

   int      index = *((int *) (arg));
   int      ii, jj, px1, py1, qx, qy;
   float    *pix1, *pix3, pix9[3], pix9f[3];
   float    red, green, blue;
   float    Btran, Etran, Vtran, Ttran;
   float    px0, py0, px2, py2;
   float    ww1, hh1, ww2, hh2, edgedist, blendist;
   float    sinT, cosT;
   uint8    *Vtransp;
   PXM      *pxm2, *pxm2f;

   for (py1 = updylo+index; py1 < updyhi; py1 += NWT)                      //  loop layout image pixels
   for (px1 = updxlo; px1 < updxhi; px1++)                                 //    within update region
   {
      pix1 = PXMpix(E1pxm,px1,py1);                                        //  input image pixel (layout/background)
      pix3 = PXMpix(E3pxm,px1,py1);                                        //  output pixel (layout + overlay images)
      
      red = pix1[0];                                                       //  start with layout image pixel
      green = pix1[1];
      blue = pix1[2];
      
      for (ii = 0; ii < Nimage; ii++)                                      //  loop overlay images from bottom to top
      {
         if (image[ii].pxhi < updxlo) continue;                            //  no overlap with update region
         if (image[ii].pxlo > updxhi) continue;
         if (image[ii].pyhi < updylo) continue;
         if (image[ii].pylo > updyhi) continue;
         px0 = image[ii].px0;                                              //  overlay image position and rotation
         py0 = image[ii].py0;                                              //    in layout image space
         sinT = image[ii].sinT;
         cosT = image[ii].cosT;
         ww2 = image[ii].ww2;                                              //  image size at scale
         hh2 = image[ii].hh2;
         px2 = (px1-px0) * cosT + (py1-py0) * sinT;                        //  overlay image pixel for layout pixel
         py2 = (py1-py0) * cosT - (px1-px0) * sinT;
         if (px2 < 0 || px2 > ww2-1) continue;                             //  layout pixel not in this image
         if (py2 < 0 || py2 > hh2-1) continue;
         
         pxm2 = image[ii].pxm2;                                            //  overlay image virt. pixel
         if (! vpixel(pxm2,px2,py2,pix9)) continue;                        //  beyond the edge

         pxm2f = image[ii].pxm2f;                                          //  pix9f = corresp. info pixel
         if (pxm2f) vpixel(pxm2f,px2,py2,pix9f);                           //    if a select area image

         Btran = image[ii].Btransp;                                        //  image base transparency 0...1
         Etran = Vtran = 0;                                                //  assume no edge or var. transparency

         if (pxm2f && pix9f[0] == 0) Etran = 1.0;                          //  select area image, outside pixel   v.15.02

         else if (image[ii].Fblend)                                        //  one or more edge blend is active   v.15.02
         {
            Etran = 0;                                                     //  edge transparency

            blendist = image[ii].Lbdist;                                   //  left edge blend distance
            if (blendist) {                                                //  active
               edgedist = px2;                                             //  pixel distance from left edge
               if (edgedist < blendist)                                    //  within blend distance
                  Etran = 1.0 - edgedist/blendist;                         //  edge transparency, 1...0
            }
            
            blendist = image[ii].Rbdist;                                   //  right edge
            if (blendist) {
               edgedist = ww2 - px2;
               if (edgedist < blendist)
                  Etran = 1 - (1 - Etran) * edgedist/blendist;
            }
            
            blendist = image[ii].Tbdist;                                   //  top edge
            if (blendist) {
               edgedist = py2;
               if (edgedist < blendist)
                  Etran = 1 - (1 - Etran) * edgedist/blendist;
            }
            
            blendist = image[ii].Bbdist;                                   //  bottom edge
            if (blendist) {
               edgedist = hh2 - py2;
               if (edgedist < blendist)
                  Etran = 1 - (1 - Etran) * edgedist/blendist;
            }
         }

         Vtransp = image[ii].Vtransp;
         if (Vtransp)                                                      //  var. transparency (px2,py2)
         {
            ww1 = image[ii].ww1;                                           //  image size at 1x scale
            hh1 = image[ii].hh1;
            qx = px2 / image[ii].scale;                                    //  position in 1x image
            qy = py2 / image[ii].scale;
            if (qx < ww1 && qy < hh1) {                                    //  position is within image
               jj = qy * ww1 + qx;
               Vtran = Vtransp[jj] / 255.0;                                //  var. transparency 0...1
            }
         }
         
         Ttran = 1 - (1 - Btran) * (1 - Etran) * (1 - Vtran);              //  overlay image pixel total transparency
         
         red = red * Ttran + (1 - Ttran) * pix9[0];                        //  combined color through this image
         green = green * Ttran + (1 - Ttran) * pix9[1];
         blue = blue * Ttran + (1 - Ttran) * pix9[2];
      }

      pix3[0] = red;                                                       //  output pixel
      pix3[1] = green;
      pix3[2] = blue;
   }
   
   exit_wthread();                                                         //  exit thread
   return 0;                                                               //  not executed, avoid gcc warning
}


void * mashup::E3update_wthreadtext(void *arg)                             //  worker thread for text images
{
   using namespace mashup;

   int      index = *((int *) (arg));
   int      ii, px, py;
   int      px0, py0, px3, py3;
   float    *pix1, *pix2, *pix3;
   float    e3part;
   PXM      *pxm_text, *pxm_transp;

   for (ii = 0; ii < Ntext; ii++)                                          //  update overlapping text images
   {
      if (ii % NWT != index) continue;                                     //  divide text among threads

      if (text[ii].pxhi < updxlo) continue;                                //  no overlap with update region
      if (text[ii].pxlo > updxhi) continue;
      if (text[ii].pyhi < updylo) continue;
      if (text[ii].pylo > updyhi) continue;

      pxm_text = text[ii].attr.pxm_text;
      pxm_transp = text[ii].attr.pxm_transp;
      px0 = text[ii].px0;                                                  //  overlay image position in layout space
      py0 = text[ii].py0;
      
      for (py = 0; py < pxm_text->hh; py++)                                //  write text over image
      for (px = 0; px < pxm_text->ww; px++)
      {
         px3 = px0 + px;
         py3 = py0 + py;
         if (px3 < 0 || px3 >= E3pxm->ww) continue;                        //  omit parts beyond edges
         if (py3 < 0 || py3 >= E3pxm->hh) continue;
         pix1 = PXMpix(pxm_text,px,py);                                    //  copy-from text pixel
         pix2 = PXMpix(pxm_transp,px,py);                                  //  copy-from transparency
         pix3 = PXMpix(E3pxm,px3,py3);                                     //  copy-to image pixel
         e3part = pix2[0] / 255.0;                                         //  image part visible through text
         pix3[0] = pix1[0] + e3part * pix3[0];                             //  combine text part + image part
         pix3[1] = pix1[1] + e3part * pix3[1];
         pix3[2] = pix1[2] + e3part * pix3[2];
      }
   }

   exit_wthread();                                                         //  exit thread
   return 0;                                                               //  not executed, avoid gcc warning
}


void * mashup::E3update_wthreadline(void *arg)                             //  worker thread for line/arrow images
{                                                                          //    v.14.07
   using namespace mashup;

   int      index = *((int *) (arg));
   int      ii, px, py;
   int      px0, py0, px3, py3;
   float    *pix1, *pix2, *pix3;
   float    e3part;
   PXM      *pxm_line, *pxm_transp;

   for (ii = 0; ii < Nline; ii++)                                          //  update overlapping line images
   {
      if (ii % NWT != index) continue;                                     //  divide lines among threads

      if (line[ii].pxhi < updxlo) continue;                                //  no overlap with update region
      if (line[ii].pxlo > updxhi) continue;
      if (line[ii].pyhi < updylo) continue;
      if (line[ii].pylo > updyhi) continue;

      pxm_line = line[ii].attr.pxm_line;
      pxm_transp = line[ii].attr.pxm_transp;
      px0 = line[ii].px0;                                                  //  overlay image position in layout space
      py0 = line[ii].py0;

      for (py = 0; py < pxm_line->hh; py++)                                //  write line over image
      for (px = 0; px < pxm_line->ww; px++)
      {
         px3 = px0 + px;
         py3 = py0 + py;
         if (px3 < 0 || px3 >= E3pxm->ww) continue;                        //  omit parts beyond edges
         if (py3 < 0 || py3 >= E3pxm->hh) continue;
         pix1 = PXMpix(pxm_line,px,py);                                    //  copy-from line pixel
         pix2 = PXMpix(pxm_transp,px,py);                                  //  copy-from transparency
         pix3 = PXMpix(E3pxm,px3,py3);                                     //  copy-to image pixel
         e3part = pix2[0] / 255.0;                                         //  image part visible through line
         pix3[0] = pix1[0] + e3part * pix3[0];                             //  combine line part + image part
         pix3[1] = pix1[1] + e3part * pix3[1];
         pix3[2] = pix1[2] + e3part * pix3[2];
      }
   }

   exit_wthread();                                                         //  exit thread
   return 0;                                                               //  not executed, avoid gcc warning
}



